"""Mode selection and setup screens."""

from typing import Optional

from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical, VerticalScroll
from textual.screen import ModalScreen
from textual.widgets import (
    Button,
    Checkbox,
    Input,
    Label,
    ListItem,
    ListView,
    Select,
    Static,
)

from ..modes import (
    ContestConfig,
    ContestMode,
    FieldDayConfig,
    FieldDayMode,
    ModeType,
    POTAConfig,
    POTAMode,
)
from ..modes.fieldday import ARRL_SECTIONS


class ModeSelectScreen(ModalScreen[Optional[ModeType | str]]):
    """Screen for selecting an operating mode."""

    CSS = """
    ModeSelectScreen {
        align: center middle;
    }

    ModeSelectScreen > Vertical {
        width: 60;
        height: auto;
        border: thick $primary;
        background: $surface;
        padding: 1;
    }

    .mode-title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
    }

    .mode-list {
        height: auto;
        max-height: 20;
        border: solid $primary;
        margin-bottom: 1;
    }

    .mode-description {
        height: auto;
        padding: 1;
        background: $surface-darken-1;
        margin-bottom: 1;
    }

    .mode-buttons {
        height: 3;
        align: center middle;
    }

    .mode-buttons Button {
        margin: 0 1;
    }
    """

    MODES = [
        (ModeType.GENERAL, "General Logging", "Standard QSO logging without special modes"),
        (ModeType.CONTEST, "Contest", "Contest logging with serial numbers and scoring"),
        (ModeType.POTA, "POTA Activation", "Activate a park - you're at the park making contacts"),
        ("pota_hunter", "POTA Hunter", "Hunt park activators - log contacts with stations at parks"),
        (ModeType.FIELDDAY, "ARRL Field Day", "Field Day with class/section exchange"),
    ]

    def __init__(self) -> None:
        super().__init__()
        self._selected_mode: Optional[ModeType | str] = ModeType.GENERAL

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static("Select Operating Mode", classes="mode-title")

            items = []
            for mode_type, name, _ in self.MODES:
                # Handle both ModeType enum and string values
                mode_value = mode_type.value if isinstance(mode_type, ModeType) else mode_type
                items.append(ListItem(Label(f"{name}"), id=f"mode_{mode_value}"))

            yield ListView(*items, id="mode-list", classes="mode-list")

            yield Static(self.MODES[0][2], id="mode-desc", classes="mode-description")

            with Horizontal(classes="mode-buttons"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button("Select", variant="primary", id="select")

    def on_mount(self) -> None:
        """Focus the list."""
        self.query_one(ListView).focus()

    @on(ListView.Highlighted)
    def _on_highlighted(self, event: ListView.Highlighted) -> None:
        """Update description when selection changes."""
        if event.item:
            item_id = event.item.id or ""
            mode_value = item_id.replace("mode_", "")
            for mode_type, name, desc in self.MODES:
                # Handle both ModeType enum and string values
                check_value = mode_type.value if isinstance(mode_type, ModeType) else mode_type
                if check_value == mode_value:
                    self._selected_mode = mode_type
                    self.query_one("#mode-desc", Static).update(desc)
                    break

    @on(ListView.Selected)
    def _on_selected(self, event: ListView.Selected) -> None:
        """Handle double-click selection."""
        self.dismiss(self._selected_mode)

    @on(Button.Pressed, "#cancel")
    def _on_cancel(self) -> None:
        self.dismiss(None)

    @on(Button.Pressed, "#select")
    def _on_select(self) -> None:
        self.dismiss(self._selected_mode)


class ContestSetupScreen(ModalScreen[Optional[ContestMode]]):
    """Screen for setting up a contest."""

    CSS = """
    ContestSetupScreen {
        align: center middle;
    }

    ContestSetupScreen > Vertical {
        width: 70;
        height: auto;
        max-height: 90%;
        border: thick $primary;
        background: $surface;
        padding: 1;
    }

    .setup-title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
        color: $accent;
    }

    .setup-section {
        height: auto;
        margin-bottom: 1;
        padding: 1;
        border: solid $secondary;
    }

    .section-title {
        text-style: bold;
        margin-bottom: 1;
    }

    .setup-row {
        height: 3;
        margin-bottom: 1;
    }

    .setup-row Label {
        width: 18;
        content-align: right middle;
        padding-right: 1;
    }

    .setup-row Input, .setup-row Select {
        width: 1fr;
    }

    .setup-buttons {
        height: 3;
        align: center middle;
        margin-top: 1;
    }

    .setup-buttons Button {
        margin: 0 1;
    }
    """

    EXCHANGE_FORMATS = [
        ("RST + Serial", "RST+SERIAL"),
        ("RST + Zone", "RST+ZONE"),
        ("RST + State/Province", "RST+STATE"),
        ("RST + Name", "RST+NAME"),
        ("RST + Serial + Zone", "RST+SERIAL+ZONE"),
        ("RST + Name + State", "RST+NAME+STATE"),
    ]

    def __init__(self, my_callsign: str = "", my_exchange: str = "") -> None:
        super().__init__()
        self._my_callsign = my_callsign
        self._my_exchange = my_exchange

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static("Contest Setup", classes="setup-title")

            with VerticalScroll():
                with Vertical(classes="setup-section"):
                    yield Static("Contest Information", classes="section-title")

                    with Horizontal(classes="setup-row"):
                        yield Label("Contest Name:")
                        yield Input(placeholder="e.g., CQ WW DX SSB", id="contest_name")

                    with Horizontal(classes="setup-row"):
                        yield Label("Contest ID:")
                        yield Input(placeholder="e.g., CQ-WW-SSB", id="contest_id")

                    with Horizontal(classes="setup-row"):
                        yield Label("Exchange Format:")
                        yield Select(
                            [(name, value) for name, value in self.EXCHANGE_FORMATS],
                            value="RST+SERIAL",
                            id="exchange_format",
                        )

                with Vertical(classes="setup-section"):
                    yield Static("My Station", classes="section-title")

                    with Horizontal(classes="setup-row"):
                        yield Label("My Callsign:")
                        yield Input(value=self._my_callsign, id="my_callsign")

                    with Horizontal(classes="setup-row"):
                        yield Label("My Exchange:")
                        yield Input(
                            value=self._my_exchange,
                            placeholder="e.g., 05 (zone) or NJ (state)",
                            id="my_exchange",
                        )

                    with Horizontal(classes="setup-row"):
                        yield Label("Starting Serial:")
                        yield Input(value="1", id="starting_serial")

                    with Horizontal(classes="setup-row"):
                        yield Label("Power:")
                        yield Select(
                            [("High (>150W)", "HIGH"), ("Low (<=150W)", "LOW"), ("QRP (<=5W)", "QRP")],
                            value="HIGH",
                            id="power",
                        )

            with Horizontal(classes="setup-buttons"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button("Start Contest", variant="primary", id="start")

    @on(Button.Pressed, "#cancel")
    def _on_cancel(self) -> None:
        self.dismiss(None)

    @on(Button.Pressed, "#start")
    def _on_start(self) -> None:
        try:
            config = ContestConfig(
                name=self.query_one("#contest_name", Input).value.strip(),
                contest_name=self.query_one("#contest_name", Input).value.strip(),
                contest_id=self.query_one("#contest_id", Input).value.strip(),
                exchange_format=self.query_one("#exchange_format", Select).value or "RST+SERIAL",
                my_callsign=self.query_one("#my_callsign", Input).value.strip().upper(),
                my_exchange=self.query_one("#my_exchange", Input).value.strip(),
                starting_serial=int(self.query_one("#starting_serial", Input).value or "1"),
                power=self.query_one("#power", Select).value or "HIGH",
            )
            mode = ContestMode(config)
            self.dismiss(mode)
        except ValueError as e:
            self.notify(f"Invalid input: {e}", severity="error")


class POTASetupScreen(ModalScreen[Optional[POTAMode]]):
    """Screen for setting up a POTA activation."""

    CSS = """
    POTASetupScreen {
        align: center middle;
    }

    POTASetupScreen > Vertical {
        width: 65;
        height: auto;
        border: thick $primary;
        background: $surface;
        padding: 1;
    }

    .setup-title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
        color: $accent;
    }

    .setup-section {
        height: auto;
        margin-bottom: 1;
        padding: 1;
        border: solid $secondary;
    }

    .section-title {
        text-style: bold;
        margin-bottom: 1;
    }

    .setup-row {
        height: 3;
        margin-bottom: 1;
    }

    .setup-row Label {
        width: 18;
        content-align: right middle;
        padding-right: 1;
    }

    .setup-row Input, .setup-row Select {
        width: 1fr;
    }

    .setup-buttons {
        height: 3;
        align: center middle;
        margin-top: 1;
    }

    .setup-buttons Button {
        margin: 0 1;
    }

    .help-text {
        color: $text-muted;
        text-style: italic;
        margin-top: 1;
    }
    """

    def __init__(self, my_callsign: str = "", my_state: str = "", my_grid: str = "") -> None:
        super().__init__()
        self._my_callsign = my_callsign
        self._my_state = my_state
        self._my_grid = my_grid

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static("Parks on the Air Setup", classes="setup-title")

            with Vertical(classes="setup-section"):
                yield Static("Activator Information", classes="section-title")

                with Horizontal(classes="setup-row"):
                    yield Label("My Callsign:")
                    yield Input(value=self._my_callsign, id="my_callsign")

                with Horizontal(classes="setup-row"):
                    yield Label("Park Reference:")
                    yield Input(placeholder="e.g., K-1234", id="my_park")

                with Horizontal(classes="setup-row"):
                    yield Label("Additional Parks:")
                    yield Input(placeholder="e.g., K-5678, K-9012 (for 2-fer)", id="additional_parks")

                with Horizontal(classes="setup-row"):
                    yield Label("State/Province:")
                    yield Input(value=self._my_state, id="my_state")

                with Horizontal(classes="setup-row"):
                    yield Label("Grid Square:")
                    yield Input(value=self._my_grid, id="my_grid")

                yield Static(
                    "Minimum 10 contacts required for a valid activation",
                    classes="help-text",
                )

            with Horizontal(classes="setup-buttons"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button("Start Activation", variant="primary", id="start")

    @on(Button.Pressed, "#cancel")
    def _on_cancel(self) -> None:
        self.dismiss(None)

    @on(Button.Pressed, "#start")
    def _on_start(self) -> None:
        try:
            additional = self.query_one("#additional_parks", Input).value.strip()
            additional_parks = [p.strip().upper() for p in additional.split(",") if p.strip()]

            config = POTAConfig(
                name="POTA Activation",
                my_callsign=self.query_one("#my_callsign", Input).value.strip().upper(),
                my_park=self.query_one("#my_park", Input).value.strip().upper(),
                additional_parks=additional_parks,
                my_state=self.query_one("#my_state", Input).value.strip().upper(),
                my_grid=self.query_one("#my_grid", Input).value.strip().upper(),
                is_activator=True,
            )
            mode = POTAMode(config)
            self.dismiss(mode)
        except ValueError as e:
            self.notify(f"Invalid input: {e}", severity="error")


class POTAHunterSetupScreen(ModalScreen[Optional[POTAMode]]):
    """Screen for setting up POTA hunting mode."""

    CSS = """
    POTAHunterSetupScreen {
        align: center middle;
    }

    POTAHunterSetupScreen > Vertical {
        width: 65;
        height: auto;
        border: thick $primary;
        background: $surface;
        padding: 1;
    }

    .setup-title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
        color: $accent;
    }

    .setup-section {
        height: auto;
        margin-bottom: 1;
        padding: 1;
        border: solid $secondary;
    }

    .section-title {
        text-style: bold;
        margin-bottom: 1;
    }

    .setup-row {
        height: 3;
        margin-bottom: 1;
    }

    .setup-row Label {
        width: 18;
        content-align: right middle;
        padding-right: 1;
    }

    .setup-row Input, .setup-row Select {
        width: 1fr;
    }

    .setup-buttons {
        height: 3;
        align: center middle;
        margin-top: 1;
    }

    .setup-buttons Button {
        margin: 0 1;
    }

    .help-text {
        color: $text-muted;
        text-style: italic;
        margin-top: 1;
    }
    """

    def __init__(self, my_callsign: str = "", my_state: str = "", my_grid: str = "") -> None:
        super().__init__()
        self._my_callsign = my_callsign
        self._my_state = my_state
        self._my_grid = my_grid

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static("POTA Hunter Setup", classes="setup-title")

            with Vertical(classes="setup-section"):
                yield Static("Hunter Information", classes="section-title")

                with Horizontal(classes="setup-row"):
                    yield Label("My Callsign:")
                    yield Input(value=self._my_callsign, id="my_callsign")

                with Horizontal(classes="setup-row"):
                    yield Label("State/Province:")
                    yield Input(value=self._my_state, id="my_state")

                with Horizontal(classes="setup-row"):
                    yield Label("Grid Square:")
                    yield Input(value=self._my_grid, id="my_grid")

                yield Static(
                    "As a hunter, you'll log contacts with park activators. "
                    "Enter their park reference in the exchange.",
                    classes="help-text",
                )

            with Horizontal(classes="setup-buttons"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button("Start Hunting", variant="primary", id="start")

    @on(Button.Pressed, "#cancel")
    def _on_cancel(self) -> None:
        self.dismiss(None)

    @on(Button.Pressed, "#start")
    def _on_start(self) -> None:
        try:
            config = POTAConfig(
                name="POTA Hunter",
                my_callsign=self.query_one("#my_callsign", Input).value.strip().upper(),
                my_park="",  # Hunters don't have a park
                additional_parks=[],
                my_state=self.query_one("#my_state", Input).value.strip().upper(),
                my_grid=self.query_one("#my_grid", Input).value.strip().upper(),
                is_activator=False,  # This is hunter mode
            )
            mode = POTAMode(config)
            self.dismiss(mode)
        except ValueError as e:
            self.notify(f"Invalid input: {e}", severity="error")


class FieldDaySetupScreen(ModalScreen[Optional[FieldDayMode]]):
    """Screen for setting up Field Day."""

    CSS = """
    FieldDaySetupScreen {
        align: center middle;
    }

    FieldDaySetupScreen > Vertical {
        width: 70;
        height: auto;
        max-height: 90%;
        border: thick $primary;
        background: $surface;
        padding: 1;
    }

    .setup-title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
        color: $accent;
    }

    .setup-section {
        height: auto;
        margin-bottom: 1;
        padding: 1;
        border: solid $secondary;
    }

    .section-title {
        text-style: bold;
        margin-bottom: 1;
    }

    .setup-row {
        height: 3;
        margin-bottom: 1;
    }

    .setup-row Label {
        width: 18;
        content-align: right middle;
        padding-right: 1;
    }

    .setup-row Input, .setup-row Select {
        width: 1fr;
    }

    .checkbox-row {
        height: 2;
    }

    .setup-buttons {
        height: 3;
        align: center middle;
        margin-top: 1;
    }

    .setup-buttons Button {
        margin: 0 1;
    }
    """

    FD_CLASSES = [
        "1A", "2A", "3A", "4A", "5A", "6A", "7A", "8A", "9A", "10A",
        "1B", "2B", "1C", "1D", "2D", "1E", "2E", "1F",
    ]

    def __init__(self, my_callsign: str = "") -> None:
        super().__init__()
        self._my_callsign = my_callsign

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static("ARRL Field Day Setup", classes="setup-title")

            with VerticalScroll():
                with Vertical(classes="setup-section"):
                    yield Static("Station Information", classes="section-title")

                    with Horizontal(classes="setup-row"):
                        yield Label("Callsign:")
                        yield Input(value=self._my_callsign, id="my_callsign")

                    with Horizontal(classes="setup-row"):
                        yield Label("Club Name:")
                        yield Input(placeholder="e.g., Example ARC", id="club_name")

                    with Horizontal(classes="setup-row"):
                        yield Label("Class:")
                        yield Select(
                            [(c, c) for c in self.FD_CLASSES],
                            value="3A",
                            id="my_class",
                        )

                    with Horizontal(classes="setup-row"):
                        yield Label("ARRL Section:")
                        yield Select(
                            [(s, s) for s in sorted(ARRL_SECTIONS)],
                            value="SNJ",
                            id="my_section",
                        )

                    with Horizontal(classes="setup-row"):
                        yield Label("Power Level:")
                        yield Select(
                            [("High (>150W)", "HIGH"), ("Low (<=150W)", "LOW"), ("QRP (<=5W)", "QRP")],
                            value="LOW",
                            id="power_level",
                        )

                with Vertical(classes="setup-section"):
                    yield Static("Bonus Points", classes="section-title")

                    yield Checkbox("100% Emergency Power", id="emergency_power")
                    yield Checkbox("Media Publicity", id="media_publicity")
                    yield Checkbox("Public Location", id="public_location")
                    yield Checkbox("Public Information Table", id="public_info_table")
                    yield Checkbox("Youth Participation", id="youth_participation")
                    yield Checkbox("Web Submission", value=True, id="web_submission")
                    yield Checkbox("Educational Activity", id="educational_activity")
                    yield Checkbox("Safety Officer", id="safety_officer")

            with Horizontal(classes="setup-buttons"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button("Start Field Day", variant="primary", id="start")

    @on(Button.Pressed, "#cancel")
    def _on_cancel(self) -> None:
        self.dismiss(None)

    @on(Button.Pressed, "#start")
    def _on_start(self) -> None:
        try:
            config = FieldDayConfig(
                name="ARRL Field Day",
                my_callsign=self.query_one("#my_callsign", Input).value.strip().upper(),
                club_name=self.query_one("#club_name", Input).value.strip(),
                my_class=self.query_one("#my_class", Select).value or "3A",
                my_section=self.query_one("#my_section", Select).value or "SNJ",
                power_level=self.query_one("#power_level", Select).value or "LOW",
                emergency_power=self.query_one("#emergency_power", Checkbox).value,
                media_publicity=self.query_one("#media_publicity", Checkbox).value,
                public_location=self.query_one("#public_location", Checkbox).value,
                public_info_table=self.query_one("#public_info_table", Checkbox).value,
                youth_participation=self.query_one("#youth_participation", Checkbox).value,
                web_submission=self.query_one("#web_submission", Checkbox).value,
                educational_activity=self.query_one("#educational_activity", Checkbox).value,
                safety_officer=self.query_one("#safety_officer", Checkbox).value,
            )
            mode = FieldDayMode(config)
            self.dismiss(mode)
        except ValueError as e:
            self.notify(f"Invalid input: {e}", severity="error")



"""DX Cluster spots service with telnet and web API support."""

import asyncio
import logging
import re
from datetime import datetime, timezone
from typing import Optional

import httpx

from ..models import Spot, SpotSource

logger = logging.getLogger(__name__)


class DXClusterError(Exception):
    """Error with DX cluster connection or fetching."""

    pass


class DXClusterService:
    """Service for fetching DX cluster spots via telnet or web API."""

    # Default telnet cluster nodes
    DEFAULT_HOST = "dxc.nc7j.com"
    DEFAULT_PORT = 7373

    # Web API endpoint (HamQTH DX Cluster)
    WEB_API_URL = "https://www.hamqth.com/dxc_csv.php"

    # Regex to parse DX spot lines
    # Format: DX de W1ABC:     14250.0  K2XYZ        CQ CQ CQ              1845Z
    SPOT_PATTERN = re.compile(
        r"DX de\s+(\S+):\s*(\d+\.?\d*)\s+(\S+)\s*(.*?)\s+(\d{4})Z",
        re.IGNORECASE,
    )

    TIMEOUT = 10.0
    TELNET_TIMEOUT = 30.0

    def __init__(
        self,
        host: str = DEFAULT_HOST,
        port: int = DEFAULT_PORT,
        callsign: str = "",
    ) -> None:
        self.host = host
        self.port = port
        self.callsign = callsign
        self._http_client: Optional[httpx.AsyncClient] = None
        self._telnet_reader: Optional[asyncio.StreamReader] = None
        self._telnet_writer: Optional[asyncio.StreamWriter] = None
        self._spots_buffer: list[Spot] = []
        self._connected = False
        self._receive_task: Optional[asyncio.Task] = None

    async def _get_http_client(self) -> httpx.AsyncClient:
        """Get or create the HTTP client."""
        if self._http_client is None or self._http_client.is_closed:
            self._http_client = httpx.AsyncClient(
                timeout=self.TIMEOUT,
                headers={"User-Agent": "TermLogger/0.1.0"},
            )
        return self._http_client

    async def close(self) -> None:
        """Close all connections."""
        await self.disconnect_telnet()
        if self._http_client and not self._http_client.is_closed:
            await self._http_client.aclose()
            self._http_client = None

    # -------------------------------------------------------------------------
    # Telnet Methods
    # -------------------------------------------------------------------------

    async def connect_telnet(self) -> bool:
        """
        Connect to the DX cluster via telnet.

        Returns:
            True if connected successfully
        """
        if not self.callsign:
            raise DXClusterError("Callsign required for telnet connection")

        try:
            self._telnet_reader, self._telnet_writer = await asyncio.wait_for(
                asyncio.open_connection(self.host, self.port),
                timeout=self.TELNET_TIMEOUT,
            )

            # Read initial prompt and send callsign
            await asyncio.sleep(0.5)
            initial = await self._read_until_prompt()
            logger.debug(f"Telnet initial: {initial}")

            # Send callsign to login
            self._telnet_writer.write(f"{self.callsign}\n".encode())
            await self._telnet_writer.drain()

            # Read login response
            await asyncio.sleep(0.5)
            response = await self._read_until_prompt()
            logger.debug(f"Telnet login response: {response}")

            self._connected = True

            # Start background task to receive spots
            self._receive_task = asyncio.create_task(self._receive_spots_loop())

            logger.info(f"Connected to DX cluster {self.host}:{self.port}")
            return True

        except asyncio.TimeoutError:
            raise DXClusterError(f"Timeout connecting to {self.host}:{self.port}")
        except Exception as e:
            raise DXClusterError(f"Failed to connect to DX cluster: {e}") from e

    async def disconnect_telnet(self) -> None:
        """Disconnect from the telnet cluster."""
        self._connected = False

        if self._receive_task:
            self._receive_task.cancel()
            try:
                await self._receive_task
            except asyncio.CancelledError:
                pass
            self._receive_task = None

        if self._telnet_writer:
            try:
                self._telnet_writer.write(b"bye\n")
                await self._telnet_writer.drain()
                self._telnet_writer.close()
                await self._telnet_writer.wait_closed()
            except Exception:
                pass
            self._telnet_writer = None
            self._telnet_reader = None

        logger.info("Disconnected from DX cluster")

    async def _read_until_prompt(self) -> str:
        """Read from telnet until a prompt or timeout."""
        if not self._telnet_reader:
            return ""

        try:
            data = await asyncio.wait_for(
                self._telnet_reader.read(4096),
                timeout=5.0,
            )
            return data.decode("utf-8", errors="ignore")
        except asyncio.TimeoutError:
            return ""

    async def _receive_spots_loop(self) -> None:
        """Background loop to receive and parse spots."""
        while self._connected and self._telnet_reader:
            try:
                data = await asyncio.wait_for(
                    self._telnet_reader.readline(),
                    timeout=60.0,
                )
                line = data.decode("utf-8", errors="ignore").strip()

                if line:
                    spot = self._parse_spot_line(line)
                    if spot:
                        self._spots_buffer.append(spot)
                        # Keep buffer size reasonable
                        if len(self._spots_buffer) > 100:
                            self._spots_buffer = self._spots_buffer[-100:]

            except asyncio.TimeoutError:
                # Send a keepalive
                if self._telnet_writer:
                    try:
                        self._telnet_writer.write(b"\n")
                        await self._telnet_writer.drain()
                    except Exception:
                        break
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"Error receiving spots: {e}")
                break

        self._connected = False

    def _parse_spot_line(self, line: str) -> Optional[Spot]:
        """Parse a DX spot line from telnet."""
        match = self.SPOT_PATTERN.search(line)
        if not match:
            return None

        try:
            spotter = match.group(1).upper().rstrip(":")
            freq_str = match.group(2)
            callsign = match.group(3).upper()
            comment = match.group(4).strip()
            time_str = match.group(5)

            # Parse frequency (could be in kHz or MHz)
            frequency = float(freq_str)
            if frequency > 1000:
                # Assume kHz, convert to MHz
                frequency = frequency / 1000

            # Parse time (HHMM format)
            now = datetime.now(timezone.utc)
            try:
                hour = int(time_str[:2])
                minute = int(time_str[2:])
                spot_time = now.replace(hour=hour, minute=minute, second=0, microsecond=0)
            except (ValueError, IndexError):
                spot_time = now

            # Try to extract mode from comment
            mode = self._extract_mode(comment)

            return Spot(
                callsign=callsign,
                frequency=frequency,
                mode=mode,
                spotter=spotter,
                comment=comment,
                time=spot_time,
                source=SpotSource.DX_CLUSTER,
            )

        except Exception as e:
            logger.debug(f"Failed to parse spot line '{line}': {e}")
            return None

    def _extract_mode(self, comment: str) -> Optional[str]:
        """Try to extract operating mode from spot comment."""
        comment_upper = comment.upper()
        modes = ["FT8", "FT4", "CW", "SSB", "RTTY", "PSK", "JS8", "FM", "AM"]
        for mode in modes:
            if mode in comment_upper:
                return mode
        return None

    def get_telnet_spots(self, limit: int = 50) -> list[Spot]:
        """
        Get spots from the telnet buffer.

        Args:
            limit: Maximum number of spots to return

        Returns:
            List of recent spots (newest first)
        """
        # Return most recent spots first
        return list(reversed(self._spots_buffer[-limit:]))

    @property
    def is_connected(self) -> bool:
        """Check if telnet connection is active."""
        return self._connected

    # -------------------------------------------------------------------------
    # Web API Methods
    # -------------------------------------------------------------------------

    async def fetch_web_spots(
        self,
        limit: int = 50,
        band: Optional[str] = None,
    ) -> list[Spot]:
        """
        Fetch spots from HamQTH DX Cluster web API.

        Args:
            limit: Maximum number of spots to return
            band: Optional band filter (e.g., "20m")

        Returns:
            List of Spot objects
        """
        try:
            client = await self._get_http_client()

            params = {
                "limit": str(limit),
            }

            if band:
                # HamQTH uses band parameter directly
                params["band"] = band.lower().rstrip("m")  # e.g., "20" for 20m

            response = await client.get(self.WEB_API_URL, params=params)
            response.raise_for_status()

            # Parse the response
            spots = self._parse_web_response(response.text)
            return spots[:limit]

        except httpx.HTTPError as e:
            logger.error(f"HTTP error fetching DX spots: {e}")
            raise DXClusterError(f"Failed to fetch DX spots: {e}") from e
        except Exception as e:
            logger.error(f"Error fetching DX spots: {e}")
            raise DXClusterError(f"Failed to fetch DX spots: {e}") from e

    def _parse_web_response(self, text: str) -> list[Spot]:
        """Parse the HamQTH DX Cluster CSV response."""
        spots = []

        # HamQTH returns caret-separated data
        # Format: spotter^frequency^callsign^comment^time^^^continent^band^country^?
        # Example: HB9DDS^7170.0^9K2KO^Nice Signal^2036 2025-12-26^^^AS^40M^Kuwait^348
        for line in text.strip().split("\n"):
            if not line:
                continue

            parts = line.split("^")
            if len(parts) < 5:
                continue

            try:
                spotter = parts[0].upper()
                freq_str = parts[1]
                callsign = parts[2].upper()
                comment = parts[3] if len(parts) > 3 else ""
                time_str = parts[4] if len(parts) > 4 else ""

                # Parse frequency (in kHz, convert to MHz)
                try:
                    frequency = float(freq_str) / 1000
                except ValueError:
                    continue

                # Parse time (format: "HHMM YYYY-MM-DD")
                try:
                    if " " in time_str:
                        time_part, date_part = time_str.split(" ", 1)
                        hour = int(time_part[:2])
                        minute = int(time_part[2:4]) if len(time_part) >= 4 else 0
                        year, month, day = map(int, date_part.split("-"))
                        spot_time = datetime(
                            year, month, day, hour, minute, tzinfo=timezone.utc
                        )
                    else:
                        spot_time = datetime.now(timezone.utc)
                except (ValueError, IndexError):
                    spot_time = datetime.now(timezone.utc)

                spots.append(
                    Spot(
                        callsign=callsign,
                        frequency=frequency,
                        mode=self._extract_mode(comment),
                        spotter=spotter,
                        comment=comment,
                        time=spot_time,
                        source=SpotSource.WEB_API,
                    )
                )

            except Exception as e:
                logger.debug(f"Failed to parse web spot: {e}")
                continue

        return spots

    # -------------------------------------------------------------------------
    # Combined Methods
    # -------------------------------------------------------------------------

    async def get_spots(
        self,
        limit: int = 50,
        use_telnet: bool = True,
        use_web: bool = True,
    ) -> list[Spot]:
        """
        Get spots from available sources.

        Args:
            limit: Maximum number of spots to return
            use_telnet: Include telnet spots if connected
            use_web: Fetch from web API if needed

        Returns:
            List of Spot objects (newest first)
        """
        spots = []

        # Get telnet spots if connected
        if use_telnet and self._connected:
            spots.extend(self.get_telnet_spots(limit))

        # Fetch from web API if enabled and we don't have enough spots
        if use_web and len(spots) < limit:
            try:
                web_spots = await self.fetch_web_spots(limit - len(spots))
                spots.extend(web_spots)
            except DXClusterError:
                pass  # Web API is optional fallback

        # Sort by time (newest first) and dedupe by callsign+frequency
        seen = set()
        unique_spots = []
        for spot in sorted(spots, key=lambda s: s.time, reverse=True):
            key = (spot.callsign, round(spot.frequency, 1))
            if key not in seen:
                seen.add(key)
                unique_spots.append(spot)

        return unique_spots[:limit]



"""SQLite database operations for TermLogger."""

import sqlite3
from contextlib import contextmanager
from datetime import datetime
from pathlib import Path
from typing import Generator, Optional

from .models import Contest, Log, LogType, Mode, QSO


class Database:
    """SQLite database manager for QSO logging."""

    def __init__(self, db_path: str):
        """Initialize database connection."""
        self.db_path = Path(db_path)
        self.db_path.parent.mkdir(parents=True, exist_ok=True)
        self._init_db()

    @contextmanager
    def _get_connection(self) -> Generator[sqlite3.Connection, None, None]:
        """Get a database connection context manager."""
        conn = sqlite3.connect(str(self.db_path))
        conn.row_factory = sqlite3.Row
        try:
            yield conn
        finally:
            conn.close()

    def _init_db(self) -> None:
        """Initialize database schema."""
        with self._get_connection() as conn:
            cursor = conn.cursor()

            # QSOs table
            cursor.execute("""
                CREATE TABLE IF NOT EXISTS qsos (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    callsign TEXT NOT NULL,
                    frequency REAL NOT NULL,
                    mode TEXT NOT NULL,
                    rst_sent TEXT DEFAULT '59',
                    rst_received TEXT DEFAULT '59',
                    datetime_utc TEXT NOT NULL,
                    notes TEXT DEFAULT '',
                    contest_id INTEGER,
                    exchange_sent TEXT,
                    exchange_received TEXT,
                    created_at TEXT DEFAULT CURRENT_TIMESTAMP,
                    FOREIGN KEY (contest_id) REFERENCES contests(id)
                )
            """)

            # Contests table
            cursor.execute("""
                CREATE TABLE IF NOT EXISTS contests (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    name TEXT NOT NULL,
                    start_time TEXT,
                    end_time TEXT,
                    exchange_format TEXT DEFAULT 'RST+SN',
                    active INTEGER DEFAULT 0
                )
            """)

            # Logs table for virtual logs
            cursor.execute("""
                CREATE TABLE IF NOT EXISTS logs (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    name TEXT NOT NULL,
                    description TEXT DEFAULT '',
                    log_type TEXT DEFAULT 'general',
                    pota_ref TEXT,
                    sota_ref TEXT,
                    contest_id INTEGER,
                    my_callsign TEXT,
                    my_gridsquare TEXT,
                    location TEXT,
                    created_at TEXT DEFAULT CURRENT_TIMESTAMP,
                    start_time TEXT,
                    end_time TEXT,
                    is_active INTEGER DEFAULT 0,
                    is_archived INTEGER DEFAULT 0,
                    FOREIGN KEY (contest_id) REFERENCES contests(id)
                )
            """)

            # Config table for key-value storage
            cursor.execute("""
                CREATE TABLE IF NOT EXISTS config (
                    key TEXT PRIMARY KEY,
                    value TEXT
                )
            """)

            # Create indexes for common queries
            cursor.execute("""
                CREATE INDEX IF NOT EXISTS idx_qsos_callsign
                ON qsos(callsign)
            """)
            cursor.execute("""
                CREATE INDEX IF NOT EXISTS idx_qsos_datetime
                ON qsos(datetime_utc)
            """)
            cursor.execute("""
                CREATE INDEX IF NOT EXISTS idx_qsos_contest
                ON qsos(contest_id)
            """)

            conn.commit()

            # Run migrations
            self._migrate_db(conn)

    def _migrate_db(self, conn: sqlite3.Connection) -> None:
        """Run database migrations for schema updates."""
        cursor = conn.cursor()

        # Check if log_id column exists in qsos table
        cursor.execute("PRAGMA table_info(qsos)")
        columns = [col[1] for col in cursor.fetchall()]

        if "log_id" not in columns:
            cursor.execute("ALTER TABLE qsos ADD COLUMN log_id INTEGER")
            cursor.execute("""
                CREATE INDEX IF NOT EXISTS idx_qsos_log
                ON qsos(log_id)
            """)
            conn.commit()

        # Create index on logs table
        cursor.execute("""
            CREATE INDEX IF NOT EXISTS idx_logs_active
            ON logs(is_active)
        """)
        cursor.execute("""
            CREATE INDEX IF NOT EXISTS idx_logs_type
            ON logs(log_type)
        """)
        conn.commit()

    def add_qso(self, qso: QSO, log_id: Optional[int] = None) -> int:
        """Add a new QSO to the database. Returns the new QSO ID."""
        # Use provided log_id or the one from the QSO object
        effective_log_id = log_id if log_id is not None else qso.log_id
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                INSERT INTO qsos (
                    callsign, frequency, mode, rst_sent, rst_received,
                    datetime_utc, notes, contest_id, exchange_sent, exchange_received,
                    log_id
                ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """,
                (
                    qso.callsign.upper(),
                    qso.frequency,
                    qso.mode.value,
                    qso.rst_sent,
                    qso.rst_received,
                    qso.datetime_utc.isoformat(),
                    qso.notes,
                    qso.contest_id,
                    qso.exchange_sent,
                    qso.exchange_received,
                    effective_log_id,
                ),
            )
            conn.commit()
            return cursor.lastrowid

    def get_qso(self, qso_id: int) -> Optional[QSO]:
        """Get a QSO by ID."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM qsos WHERE id = ?", (qso_id,))
            row = cursor.fetchone()
            return self._row_to_qso(row) if row else None

    def get_all_qsos(
        self,
        limit: int = 100,
        offset: int = 0,
        contest_id: Optional[int] = None,
        log_id: Optional[int] = None,
    ) -> list[QSO]:
        """Get QSOs with pagination, optionally filtered by contest or log."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            query = "SELECT * FROM qsos WHERE 1=1"
            params: list = []

            if contest_id is not None:
                query += " AND contest_id = ?"
                params.append(contest_id)

            if log_id is not None:
                query += " AND log_id = ?"
                params.append(log_id)

            query += " ORDER BY datetime_utc DESC LIMIT ? OFFSET ?"
            params.extend([limit, offset])

            cursor.execute(query, params)
            return [self._row_to_qso(row) for row in cursor.fetchall()]

    def get_recent_qsos(self, count: int = 50, log_id: Optional[int] = None) -> list[QSO]:
        """Get the most recent QSOs, optionally filtered by log."""
        return self.get_all_qsos(limit=count, log_id=log_id)

    def search_qsos(self, callsign: str) -> list[QSO]:
        """Search for QSOs by callsign (partial match)."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                SELECT * FROM qsos
                WHERE callsign LIKE ?
                ORDER BY datetime_utc DESC
            """,
                (f"%{callsign.upper()}%",),
            )
            return [self._row_to_qso(row) for row in cursor.fetchall()]

    def check_dupe(
        self,
        callsign: str,
        band: Optional[str] = None,
        mode: Optional[str] = None,
        contest_id: Optional[int] = None,
    ) -> bool:
        """Check if a QSO is a duplicate (same callsign, optionally same band/mode)."""
        with self._get_connection() as conn:
            cursor = conn.cursor()

            query = "SELECT COUNT(*) FROM qsos WHERE UPPER(callsign) = ?"
            params = [callsign.upper()]

            if mode:
                query += " AND mode = ?"
                params.append(mode)

            if contest_id is not None:
                query += " AND contest_id = ?"
                params.append(contest_id)

            cursor.execute(query, params)
            count = cursor.fetchone()[0]
            return count > 0

    def update_qso(self, qso: QSO) -> bool:
        """Update an existing QSO."""
        if qso.id is None:
            return False

        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                UPDATE qsos SET
                    callsign = ?, frequency = ?, mode = ?,
                    rst_sent = ?, rst_received = ?, datetime_utc = ?,
                    notes = ?, contest_id = ?, exchange_sent = ?, exchange_received = ?,
                    log_id = ?
                WHERE id = ?
            """,
                (
                    qso.callsign.upper(),
                    qso.frequency,
                    qso.mode.value,
                    qso.rst_sent,
                    qso.rst_received,
                    qso.datetime_utc.isoformat(),
                    qso.notes,
                    qso.contest_id,
                    qso.exchange_sent,
                    qso.exchange_received,
                    qso.log_id,
                    qso.id,
                ),
            )
            conn.commit()
            return cursor.rowcount > 0

    def delete_qso(self, qso_id: int) -> bool:
        """Delete a QSO by ID."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("DELETE FROM qsos WHERE id = ?", (qso_id,))
            conn.commit()
            return cursor.rowcount > 0

    def get_qso_count(
        self, contest_id: Optional[int] = None, log_id: Optional[int] = None
    ) -> int:
        """Get total QSO count, optionally filtered by contest or log."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            query = "SELECT COUNT(*) FROM qsos WHERE 1=1"
            params: list = []

            if contest_id is not None:
                query += " AND contest_id = ?"
                params.append(contest_id)

            if log_id is not None:
                query += " AND log_id = ?"
                params.append(log_id)

            cursor.execute(query, params)
            return cursor.fetchone()[0]

    def _row_to_qso(self, row: sqlite3.Row) -> QSO:
        """Convert a database row to a QSO object."""
        # Handle log_id which may not exist in older databases during migration
        log_id = None
        try:
            log_id = row["log_id"]
        except (KeyError, IndexError):
            pass

        return QSO(
            id=row["id"],
            callsign=row["callsign"],
            frequency=row["frequency"],
            mode=Mode(row["mode"]),
            rst_sent=row["rst_sent"],
            rst_received=row["rst_received"],
            datetime_utc=datetime.fromisoformat(row["datetime_utc"]),
            notes=row["notes"] or "",
            log_id=log_id,
            contest_id=row["contest_id"],
            exchange_sent=row["exchange_sent"],
            exchange_received=row["exchange_received"],
            created_at=datetime.fromisoformat(row["created_at"])
            if row["created_at"]
            else None,
        )

    # Contest methods
    def add_contest(self, contest: Contest) -> int:
        """Add a new contest. Returns the new contest ID."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                INSERT INTO contests (name, start_time, end_time, exchange_format, active)
                VALUES (?, ?, ?, ?, ?)
            """,
                (
                    contest.name,
                    contest.start_time.isoformat() if contest.start_time else None,
                    contest.end_time.isoformat() if contest.end_time else None,
                    contest.exchange_format,
                    1 if contest.active else 0,
                ),
            )
            conn.commit()
            return cursor.lastrowid

    def get_active_contest(self) -> Optional[Contest]:
        """Get the currently active contest."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM contests WHERE active = 1 LIMIT 1")
            row = cursor.fetchone()
            if row:
                return Contest(
                    id=row["id"],
                    name=row["name"],
                    start_time=datetime.fromisoformat(row["start_time"])
                    if row["start_time"]
                    else None,
                    end_time=datetime.fromisoformat(row["end_time"])
                    if row["end_time"]
                    else None,
                    exchange_format=row["exchange_format"],
                    active=bool(row["active"]),
                )
            return None

    # Log methods
    def add_log(self, log: Log) -> int:
        """Add a new log. Returns the new log ID."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                INSERT INTO logs (
                    name, description, log_type, pota_ref, sota_ref, contest_id,
                    my_callsign, my_gridsquare, location, start_time, end_time,
                    is_active, is_archived
                ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """,
                (
                    log.name,
                    log.description,
                    log.log_type.value,
                    log.pota_ref,
                    log.sota_ref,
                    log.contest_id,
                    log.my_callsign,
                    log.my_gridsquare,
                    log.location,
                    log.start_time.isoformat() if log.start_time else None,
                    log.end_time.isoformat() if log.end_time else None,
                    1 if log.is_active else 0,
                    1 if log.is_archived else 0,
                ),
            )
            conn.commit()
            return cursor.lastrowid

    def get_log(self, log_id: int) -> Optional[Log]:
        """Get a log by ID."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM logs WHERE id = ?", (log_id,))
            row = cursor.fetchone()
            if row:
                log = self._row_to_log(row)
                log.qso_count = self.get_qso_count(log_id=log_id)
                return log
            return None

    def get_all_logs(
        self, include_archived: bool = False, limit: int = 100
    ) -> list[Log]:
        """Get all logs, optionally including archived ones."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            if include_archived:
                cursor.execute(
                    "SELECT * FROM logs ORDER BY created_at DESC LIMIT ?", (limit,)
                )
            else:
                cursor.execute(
                    """
                    SELECT * FROM logs
                    WHERE is_archived = 0
                    ORDER BY created_at DESC
                    LIMIT ?
                """,
                    (limit,),
                )
            logs = [self._row_to_log(row) for row in cursor.fetchall()]
            # Add QSO counts
            for log in logs:
                log.qso_count = self.get_qso_count(log_id=log.id)
            return logs

    def get_active_log(self) -> Optional[Log]:
        """Get the currently active log."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM logs WHERE is_active = 1 LIMIT 1")
            row = cursor.fetchone()
            if row:
                log = self._row_to_log(row)
                log.qso_count = self.get_qso_count(log_id=log.id)
                return log
            return None

    def set_active_log(self, log_id: Optional[int]) -> bool:
        """Set the active log. Pass None to clear active log."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            # First, clear any existing active log
            cursor.execute("UPDATE logs SET is_active = 0 WHERE is_active = 1")
            # Then set the new active log if provided
            if log_id is not None:
                cursor.execute(
                    "UPDATE logs SET is_active = 1 WHERE id = ?", (log_id,)
                )
            conn.commit()
            return True

    def update_log(self, log: Log) -> bool:
        """Update an existing log."""
        if log.id is None:
            return False

        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                UPDATE logs SET
                    name = ?, description = ?, log_type = ?, pota_ref = ?,
                    sota_ref = ?, contest_id = ?, my_callsign = ?, my_gridsquare = ?,
                    location = ?, start_time = ?, end_time = ?, is_active = ?,
                    is_archived = ?
                WHERE id = ?
            """,
                (
                    log.name,
                    log.description,
                    log.log_type.value,
                    log.pota_ref,
                    log.sota_ref,
                    log.contest_id,
                    log.my_callsign,
                    log.my_gridsquare,
                    log.location,
                    log.start_time.isoformat() if log.start_time else None,
                    log.end_time.isoformat() if log.end_time else None,
                    1 if log.is_active else 0,
                    1 if log.is_archived else 0,
                    log.id,
                ),
            )
            conn.commit()
            return cursor.rowcount > 0

    def delete_log(self, log_id: int, reassign_qsos_to: Optional[int] = None) -> bool:
        """Delete a log. Optionally reassign QSOs to another log."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            # Reassign or nullify QSOs
            if reassign_qsos_to is not None:
                cursor.execute(
                    "UPDATE qsos SET log_id = ? WHERE log_id = ?",
                    (reassign_qsos_to, log_id),
                )
            else:
                cursor.execute(
                    "UPDATE qsos SET log_id = NULL WHERE log_id = ?", (log_id,)
                )
            # Delete the log
            cursor.execute("DELETE FROM logs WHERE id = ?", (log_id,))
            conn.commit()
            return cursor.rowcount > 0

    def archive_log(self, log_id: int) -> bool:
        """Archive a log (soft delete)."""
        with self._get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(
                "UPDATE logs SET is_archived = 1, is_active = 0 WHERE id = ?",
                (log_id,),
            )
            conn.commit()
            return cursor.rowcount > 0

    def _row_to_log(self, row: sqlite3.Row) -> Log:
        """Convert a database row to a Log object."""
        return Log(
            id=row["id"],
            name=row["name"],
            description=row["description"] or "",
            log_type=LogType(row["log_type"]),
            pota_ref=row["pota_ref"],
            sota_ref=row["sota_ref"],
            contest_id=row["contest_id"],
            my_callsign=row["my_callsign"],
            my_gridsquare=row["my_gridsquare"],
            location=row["location"],
            created_at=datetime.fromisoformat(row["created_at"])
            if row["created_at"]
            else None,
            start_time=datetime.fromisoformat(row["start_time"])
            if row["start_time"]
            else None,
            end_time=datetime.fromisoformat(row["end_time"])
            if row["end_time"]
            else None,
            is_active=bool(row["is_active"]),
            is_archived=bool(row["is_archived"]),
        )



"""Tests for spot services."""

from datetime import datetime, timezone

import pytest

from termlogger.models import Spot, SpotSource


class TestSpotModel:
    """Tests for the Spot model."""

    def test_create_spot(self):
        """Test creating a basic spot."""
        spot = Spot(
            callsign="W1ABC",
            frequency=14.250,
            mode="SSB",
            spotter="K2XYZ",
            comment="CQ CQ CQ",
            source=SpotSource.DX_CLUSTER,
        )
        assert spot.callsign == "W1ABC"
        assert spot.frequency == 14.250
        assert spot.mode == "SSB"
        assert spot.spotter == "K2XYZ"
        assert spot.source == SpotSource.DX_CLUSTER

    def test_create_pota_spot(self):
        """Test creating a POTA spot."""
        spot = Spot(
            callsign="N3ABC",
            frequency=7.250,
            mode="SSB",
            spotter="POTA",
            comment="",
            source=SpotSource.POTA,
            park_reference="K-1234",
            park_name="Acadia National Park",
        )
        assert spot.source == SpotSource.POTA
        assert spot.park_reference == "K-1234"
        assert spot.park_name == "Acadia National Park"

    def test_spot_band_property(self):
        """Test the band property derivation from frequency."""
        spot_20m = Spot(callsign="W1ABC", frequency=14.250, source=SpotSource.DX_CLUSTER)
        assert spot_20m.band is not None
        assert spot_20m.band.value == "20m"

        spot_40m = Spot(callsign="W1ABC", frequency=7.150, source=SpotSource.DX_CLUSTER)
        assert spot_40m.band is not None
        assert spot_40m.band.value == "40m"

    def test_spot_time_str(self):
        """Test the time_str property."""
        spot = Spot(
            callsign="W1ABC",
            frequency=14.250,
            time=datetime(2024, 1, 15, 14, 30, 0, tzinfo=timezone.utc),
            source=SpotSource.DX_CLUSTER,
        )
        assert spot.time_str == "14:30"

    def test_spot_info_str_pota(self):
        """Test info_str for POTA spots."""
        spot = Spot(
            callsign="W1ABC",
            frequency=14.250,
            source=SpotSource.POTA,
            park_reference="K-1234",
            park_name="Test Park",
        )
        assert "K-1234" in spot.info_str
        assert "Test Park" in spot.info_str

    def test_spot_info_str_dx(self):
        """Test info_str for DX cluster spots."""
        spot = Spot(
            callsign="W1ABC",
            frequency=14.250,
            source=SpotSource.DX_CLUSTER,
            comment="CQ Contest",
        )
        assert spot.info_str == "CQ Contest"


class TestPOTASpotService:
    """Tests for POTA spot service parsing."""

    def test_parse_pota_spot_data(self):
        """Test parsing POTA API response data."""
        from termlogger.services.pota_spots import POTASpotService

        service = POTASpotService()

        # Simulate API response data
        spot_data = {
            "activator": "W1ABC",
            "frequency": "14.250",
            "mode": "SSB",
            "reference": "K-1234",
            "parkName": "Test National Park",
            "spotTime": "2024-01-15T14:30:00Z",
            "spotter": "K2XYZ",
            "comments": "Great signals",
        }

        spot = service._parse_spot(spot_data)

        assert spot is not None
        assert spot.callsign == "W1ABC"
        assert spot.frequency == 14.250
        assert spot.mode == "SSB"
        assert spot.park_reference == "K-1234"
        assert spot.park_name == "Test National Park"
        assert spot.spotter == "K2XYZ"
        assert spot.source == SpotSource.POTA

    def test_parse_spot_invalid_frequency(self):
        """Test parsing spot with invalid frequency."""
        from termlogger.services.pota_spots import POTASpotService

        service = POTASpotService()

        spot_data = {
            "activator": "W1ABC",
            "frequency": "invalid",
            "mode": "SSB",
            "reference": "K-1234",
        }

        spot = service._parse_spot(spot_data)
        assert spot is None

    def test_parse_spot_zero_frequency(self):
        """Test parsing spot with zero frequency."""
        from termlogger.services.pota_spots import POTASpotService

        service = POTASpotService()

        spot_data = {
            "activator": "W1ABC",
            "frequency": "0",
            "mode": "SSB",
        }

        spot = service._parse_spot(spot_data)
        assert spot is None


class TestDXClusterService:
    """Tests for DX cluster service parsing."""

    def test_parse_dx_spot_line(self):
        """Test parsing a standard DX cluster spot line."""
        from termlogger.services.dx_cluster import DXClusterService

        service = DXClusterService()

        line = "DX de W1ABC:     14250.0  K2XYZ        CQ CQ CQ              1845Z"
        spot = service._parse_spot_line(line)

        assert spot is not None
        assert spot.spotter == "W1ABC"
        assert spot.callsign == "K2XYZ"
        assert abs(spot.frequency - 14.250) < 0.001  # kHz to MHz
        assert "CQ" in spot.comment
        assert spot.source == SpotSource.DX_CLUSTER

    def test_parse_dx_spot_with_mode(self):
        """Test parsing spot with mode in comment."""
        from termlogger.services.dx_cluster import DXClusterService

        service = DXClusterService()

        line = "DX de N3ABC:     7074.0  W5XYZ        FT8 -12dB             2030Z"
        spot = service._parse_spot_line(line)

        assert spot is not None
        assert spot.mode == "FT8"

    def test_parse_invalid_line(self):
        """Test parsing an invalid line returns None."""
        from termlogger.services.dx_cluster import DXClusterService

        service = DXClusterService()

        line = "This is not a spot line"
        spot = service._parse_spot_line(line)
        assert spot is None

    def test_extract_mode_from_comment(self):
        """Test mode extraction from various comments."""
        from termlogger.services.dx_cluster import DXClusterService

        service = DXClusterService()

        assert service._extract_mode("FT8 -12dB") == "FT8"
        assert service._extract_mode("CW up 2") == "CW"
        assert service._extract_mode("SSB contest") == "SSB"
        assert service._extract_mode("Working simplex") is None

    def test_parse_hamqth_web_response(self):
        """Test parsing HamQTH DX cluster web API response."""
        from termlogger.services.dx_cluster import DXClusterService

        service = DXClusterService()

        # Simulate HamQTH API response (caret-separated)
        response_text = """HB9DDS^7170.0^9K2KO^Nice Signal Mussalam.^2036 2025-12-26^^^AS^40M^Kuwait^348
ES2IPA^5357.0^SM2LKW^FT8^2035 2025-12-26^^^EU^60M^Sweden^284
AC3RA^7035.8^VE3RIA^^2035 2025-12-26^^^NA^40M^Canada^1"""

        spots = service._parse_web_response(response_text)

        assert len(spots) == 3

        # Check first spot
        assert spots[0].spotter == "HB9DDS"
        assert spots[0].callsign == "9K2KO"
        assert abs(spots[0].frequency - 7.170) < 0.001
        assert spots[0].comment == "Nice Signal Mussalam."

        # Check second spot with FT8 mode
        assert spots[1].callsign == "SM2LKW"
        assert spots[1].mode == "FT8"

        # Check third spot
        assert spots[2].callsign == "VE3RIA"
        assert spots[2].spotter == "AC3RA"



"""File picker screen for ADIF import/export."""

from pathlib import Path
from typing import Optional

from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.screen import ModalScreen
from textual.widgets import Button, DirectoryTree, Input, Label, Static


class FilePickerScreen(ModalScreen[Optional[Path]]):
    """A modal file picker screen."""

    BINDINGS = [
        ("escape", "cancel", "Cancel"),
        ("enter", "select", "Select"),
    ]

    CSS = """
    FilePickerScreen {
        align: center middle;
    }

    FilePickerScreen > Vertical {
        width: 80;
        height: 30;
        border: thick $primary;
        background: $surface;
        padding: 1;
    }

    FilePickerScreen .title {
        text-align: center;
        text-style: bold;
        height: 1;
        margin-bottom: 1;
    }

    FilePickerScreen .path-input {
        height: 3;
        margin-bottom: 1;
    }

    FilePickerScreen DirectoryTree {
        height: 1fr;
        border: solid $primary;
        margin-bottom: 1;
    }

    FilePickerScreen .button-row {
        height: 3;
        align: center middle;
    }

    FilePickerScreen Button {
        margin: 0 1;
    }
    """

    def __init__(
        self,
        title: str = "Select File",
        start_path: Optional[Path] = None,
        extensions: Optional[list[str]] = None,
        save_mode: bool = False,
        default_filename: str = "",
    ) -> None:
        """Initialize the file picker.

        Args:
            title: Title to display
            start_path: Starting directory path
            extensions: List of file extensions to filter (e.g., [".adi", ".adif"])
            save_mode: If True, allow entering a new filename
            default_filename: Default filename for save mode
        """
        super().__init__()
        self.title_text = title
        self.start_path = start_path or Path.home()
        self.extensions = extensions or []
        self.save_mode = save_mode
        self.default_filename = default_filename
        self._selected_path: Optional[Path] = None

    def compose(self) -> ComposeResult:
        """Create child widgets."""
        with Vertical():
            yield Label(self.title_text, classes="title")

            yield Input(
                value=str(self.start_path / self.default_filename)
                if self.default_filename
                else str(self.start_path),
                placeholder="Enter file path...",
                id="path-input",
                classes="path-input",
            )

            yield DirectoryTree(str(self.start_path), id="file-tree")

            with Horizontal(classes="button-row"):
                yield Button("Cancel", variant="default", id="cancel")
                yield Button(
                    "Save" if self.save_mode else "Open",
                    variant="primary",
                    id="select",
                )

    def on_mount(self) -> None:
        """Focus the path input on mount."""
        self.query_one("#path-input", Input).focus()

    @on(DirectoryTree.FileSelected)
    def _on_file_selected(self, event: DirectoryTree.FileSelected) -> None:
        """Handle file selection in the tree."""
        path = Path(event.path)

        # Filter by extension if specified
        if self.extensions:
            if path.suffix.lower() not in [ext.lower() for ext in self.extensions]:
                return

        self._selected_path = path
        self.query_one("#path-input", Input).value = str(path)

    @on(DirectoryTree.DirectorySelected)
    def _on_directory_selected(self, event: DirectoryTree.DirectorySelected) -> None:
        """Handle directory selection in the tree."""
        self.query_one("#path-input", Input).value = str(event.path)

    @on(Button.Pressed, "#cancel")
    def _on_cancel(self) -> None:
        """Handle cancel button."""
        self.dismiss(None)

    @on(Button.Pressed, "#select")
    def _on_select(self) -> None:
        """Handle select button."""
        self._do_select()

    @on(Input.Submitted, "#path-input")
    def _on_path_submitted(self, event: Input.Submitted) -> None:
        """Handle enter in path input."""
        self._do_select()

    def _do_select(self) -> None:
        """Perform the selection."""
        path_str = self.query_one("#path-input", Input).value.strip()
        if path_str:
            path = Path(path_str)

            # For save mode, allow non-existent files
            if self.save_mode:
                # Add default extension if missing
                if self.extensions and path.suffix.lower() not in [
                    ext.lower() for ext in self.extensions
                ]:
                    path = path.with_suffix(self.extensions[0])
                self.dismiss(path)
            else:
                # For open mode, file must exist
                if path.exists() and path.is_file():
                    self.dismiss(path)
                else:
                    self.notify("File not found", severity="error")
        else:
            self.notify("Please enter a file path", severity="warning")

    def action_cancel(self) -> None:
        """Cancel action."""
        self.dismiss(None)

    def action_select(self) -> None:
        """Select action."""
        self._do_select()


class ExportCompleteScreen(ModalScreen[None]):
    """Modal screen showing export completion."""

    CSS = """
    ExportCompleteScreen {
        align: center middle;
    }

    ExportCompleteScreen > Vertical {
        width: 50;
        height: 10;
        border: thick $success;
        background: $surface;
        padding: 1;
    }

    ExportCompleteScreen .message {
        text-align: center;
        height: 1fr;
        content-align: center middle;
    }

    ExportCompleteScreen Button {
        width: 100%;
    }
    """

    def __init__(self, message: str) -> None:
        super().__init__()
        self.message = message

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static(self.message, classes="message")
            yield Button("OK", variant="primary", id="ok")

    @on(Button.Pressed, "#ok")
    def _on_ok(self) -> None:
        self.dismiss(None)

    def on_key(self, event) -> None:
        if event.key in ("enter", "escape"):
            self.dismiss(None)



"""Data models for TermLogger."""

from datetime import datetime, timezone
from enum import Enum
from typing import Optional

from pydantic import BaseModel, Field


class Mode(str, Enum):
    """Amateur radio operating modes."""

    SSB = "SSB"
    CW = "CW"
    FM = "FM"
    AM = "AM"
    RTTY = "RTTY"
    PSK31 = "PSK31"
    FT8 = "FT8"
    FT4 = "FT4"
    JS8 = "JS8"
    SSTV = "SSTV"
    DIGITAL = "DIGITAL"


class Band(str, Enum):
    """Amateur radio bands."""

    BAND_160M = "160m"
    BAND_80M = "80m"
    BAND_60M = "60m"
    BAND_40M = "40m"
    BAND_30M = "30m"
    BAND_20M = "20m"
    BAND_17M = "17m"
    BAND_15M = "15m"
    BAND_12M = "12m"
    BAND_10M = "10m"
    BAND_6M = "6m"
    BAND_2M = "2m"
    BAND_70CM = "70cm"


# Band frequency ranges in MHz
BAND_FREQUENCIES = {
    Band.BAND_160M: (1.8, 2.0),
    Band.BAND_80M: (3.5, 4.0),
    Band.BAND_60M: (5.3, 5.4),
    Band.BAND_40M: (7.0, 7.3),
    Band.BAND_30M: (10.1, 10.15),
    Band.BAND_20M: (14.0, 14.35),
    Band.BAND_17M: (18.068, 18.168),
    Band.BAND_15M: (21.0, 21.45),
    Band.BAND_12M: (24.89, 24.99),
    Band.BAND_10M: (28.0, 29.7),
    Band.BAND_6M: (50.0, 54.0),
    Band.BAND_2M: (144.0, 148.0),
    Band.BAND_70CM: (420.0, 450.0),
}


def frequency_to_band(freq_mhz: float) -> Optional[Band]:
    """Convert a frequency in MHz to its corresponding band."""
    for band, (low, high) in BAND_FREQUENCIES.items():
        if low <= freq_mhz <= high:
            return band
    return None


class QSO(BaseModel):
    """A single QSO (contact) record."""

    # Core fields
    id: Optional[int] = None
    callsign: str = Field(..., min_length=1, max_length=15)
    frequency: float = Field(..., gt=0)
    mode: Mode = Mode.SSB
    rst_sent: str = Field(default="59", max_length=5)
    rst_received: str = Field(default="59", max_length=5)
    datetime_utc: datetime = Field(default_factory=datetime.utcnow)
    notes: str = Field(default="", max_length=500)

    # Log association
    log_id: Optional[int] = None  # Virtual log this QSO belongs to

    # Contest fields
    contest_id: Optional[int] = None
    exchange_sent: Optional[str] = None
    exchange_received: Optional[str] = None

    # Extended ADIF fields - Contacted station info
    name: Optional[str] = None  # Operator name
    qth: Optional[str] = None  # City/location
    state: Optional[str] = None  # State/province
    country: Optional[str] = None  # Country name
    dxcc: Optional[int] = None  # DXCC entity number
    gridsquare: Optional[str] = None  # Maidenhead grid square
    cq_zone: Optional[int] = None  # CQ zone
    itu_zone: Optional[int] = None  # ITU zone
    continent: Optional[str] = None  # Continent (NA, SA, EU, AF, AS, OC, AN)

    # Power and equipment
    tx_pwr: Optional[float] = None  # Transmit power in watts
    rx_pwr: Optional[float] = None  # Received power (if known)
    ant_az: Optional[float] = None  # Antenna azimuth
    ant_el: Optional[float] = None  # Antenna elevation

    # Special activity references
    sota_ref: Optional[str] = None  # SOTA reference (e.g., W4C/CM-001)
    pota_ref: Optional[str] = None  # POTA reference (e.g., K-1234)
    wwff_ref: Optional[str] = None  # WWFF reference
    iota: Optional[str] = None  # IOTA reference (e.g., NA-001)
    sig: Optional[str] = None  # Special interest activity
    sig_info: Optional[str] = None  # Special interest activity info

    # QSL info
    qsl_sent: Optional[str] = None  # Y, N, R, I, Q
    qsl_sent_date: Optional[datetime] = None
    qsl_rcvd: Optional[str] = None  # Y, N, R, I, Q
    qsl_rcvd_date: Optional[datetime] = None
    qsl_via: Optional[str] = None  # QSL route (direct, bureau, manager call)
    lotw_qsl_sent: Optional[str] = None  # LoTW sent status
    lotw_qsl_rcvd: Optional[str] = None  # LoTW received status
    eqsl_qsl_sent: Optional[str] = None  # eQSL sent status
    eqsl_qsl_rcvd: Optional[str] = None  # eQSL received status

    # Propagation
    prop_mode: Optional[str] = None  # Propagation mode (e.g., SAT, EME, TR)
    sat_name: Optional[str] = None  # Satellite name
    sat_mode: Optional[str] = None  # Satellite mode

    # Metadata
    operator: Optional[str] = None  # Logging operator callsign
    station_callsign: Optional[str] = None  # Station callsign if different
    my_gridsquare: Optional[str] = None  # My grid square
    my_sota_ref: Optional[str] = None  # My SOTA reference
    my_pota_ref: Optional[str] = None  # My POTA reference
    comment: Optional[str] = None  # Extended comment

    created_at: Optional[datetime] = None

    @property
    def band(self) -> Optional[Band]:
        """Get the band for this QSO based on frequency."""
        return frequency_to_band(self.frequency)

    @property
    def time_str(self) -> str:
        """Get formatted time string (HH:MM)."""
        return self.datetime_utc.strftime("%H:%M")

    @property
    def date_str(self) -> str:
        """Get formatted date string (YYYY-MM-DD)."""
        return self.datetime_utc.strftime("%Y-%m-%d")


class Contest(BaseModel):
    """Contest configuration."""

    id: Optional[int] = None
    name: str
    start_time: Optional[datetime] = None
    end_time: Optional[datetime] = None
    exchange_format: str = Field(default="RST+SN")
    active: bool = False


class LogType(str, Enum):
    """Types of virtual logs."""

    GENERAL = "general"
    POTA_ACTIVATION = "pota_activation"
    POTA_HUNTER = "pota_hunter"
    SOTA = "sota"
    CONTEST = "contest"
    FIELD_DAY = "field_day"
    DX_EXPEDITION = "dx_expedition"
    SPECIAL_EVENT = "special_event"


class Log(BaseModel):
    """A virtual log for organizing QSOs by activation, event, or session."""

    id: Optional[int] = None
    name: str = Field(..., min_length=1, max_length=100)
    description: str = Field(default="", max_length=500)
    log_type: LogType = LogType.GENERAL

    # Associated references
    pota_ref: Optional[str] = None  # POTA park reference if POTA activation
    sota_ref: Optional[str] = None  # SOTA summit reference if SOTA activation
    contest_id: Optional[int] = None  # Contest ID if contest log

    # Metadata
    my_callsign: Optional[str] = None  # Callsign used for this log
    my_gridsquare: Optional[str] = None  # Grid square for this log
    location: Optional[str] = None  # Location description

    # Timestamps
    created_at: Optional[datetime] = None
    start_time: Optional[datetime] = None  # When the activation/event started
    end_time: Optional[datetime] = None  # When the activation/event ended

    # Status
    is_active: bool = False  # Is this the currently active log?
    is_archived: bool = False  # Archived logs are hidden from normal view

    # Statistics (computed, not stored)
    qso_count: int = 0

    @property
    def display_name(self) -> str:
        """Get a display name with type indicator."""
        type_prefix = {
            LogType.GENERAL: "",
            LogType.POTA_ACTIVATION: "POTA: ",
            LogType.POTA_HUNTER: "Hunter: ",
            LogType.SOTA: "SOTA: ",
            LogType.CONTEST: "Contest: ",
            LogType.FIELD_DAY: "FD: ",
            LogType.DX_EXPEDITION: "DXped: ",
            LogType.SPECIAL_EVENT: "Event: ",
        }
        return f"{type_prefix.get(self.log_type, '')}{self.name}"

    @property
    def date_str(self) -> str:
        """Get formatted date string."""
        if self.start_time:
            return self.start_time.strftime("%Y-%m-%d")
        if self.created_at:
            return self.created_at.strftime("%Y-%m-%d")
        return ""


class CallsignLookupResult(BaseModel):
    """Result from a callsign lookup service."""

    callsign: str
    name: Optional[str] = None
    address: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    country: Optional[str] = None
    grid_square: Optional[str] = None
    latitude: Optional[float] = None
    longitude: Optional[float] = None
    qsl_via: Optional[str] = None
    email: Optional[str] = None

    @property
    def location_str(self) -> str:
        """Get a formatted location string."""
        parts = []
        if self.city:
            parts.append(self.city)
        if self.state:
            parts.append(self.state)
        if self.country:
            parts.append(self.country)
        return ", ".join(parts) if parts else "Unknown"

    @property
    def display_str(self) -> str:
        """Get a formatted display string for the UI."""
        parts = []
        if self.name:
            parts.append(self.name)
        if self.location_str != "Unknown":
            parts.append(self.location_str)
        if self.grid_square:
            parts.append(f"[{self.grid_square}]")
        return " - ".join(parts) if parts else "No information available"


class SpotSource(str, Enum):
    """Source of a spot."""

    POTA = "pota"
    DX_CLUSTER = "dxcluster"
    WEB_API = "webapi"


class Spot(BaseModel):
    """A DX or POTA spot."""

    callsign: str = Field(..., description="Spotted station callsign")
    frequency: float = Field(..., gt=0, description="Frequency in MHz")
    mode: Optional[str] = Field(default=None, description="Operating mode")
    spotter: str = Field(default="", description="Callsign of the spotter")
    comment: str = Field(default="", description="Spot comment or info")
    time: datetime = Field(default_factory=lambda: datetime.now(timezone.utc), description="Spot time UTC")
    source: SpotSource = Field(default=SpotSource.DX_CLUSTER, description="Spot source")

    # POTA-specific fields
    park_reference: Optional[str] = Field(default=None, description="POTA park reference (e.g., K-1234)")
    park_name: Optional[str] = Field(default=None, description="Park name")

    # Location info
    grid_square: Optional[str] = Field(default=None, description="Grid square")
    state: Optional[str] = Field(default=None, description="State/province")
    country: Optional[str] = Field(default=None, description="Country")

    @property
    def band(self) -> Optional[Band]:
        """Get the band for this spot based on frequency."""
        return frequency_to_band(self.frequency)

    @property
    def time_str(self) -> str:
        """Get formatted time string (HH:MM)."""
        return self.time.strftime("%H:%M")

    @property
    def info_str(self) -> str:
        """Get info string for display (park ref or comment)."""
        if self.park_reference:
            return f"{self.park_reference} {self.park_name or ''}"[:25]
        return self.comment[:25] if self.comment else ""



"""Services for TermLogger."""

from .dx_cluster import DXClusterService
from .flexradio import FlexRadioError, FlexRadioService, FlexState
from .pota_parks import Park, POTAParksError, POTAParksService
from .pota_spots import POTASpotService
from .rigctld import RigctldError, RigctldService, RigState

__all__ = [
    "POTASpotService",
    "POTAParksService",
    "POTAParksError",
    "Park",
    "DXClusterService",
    "RigctldService",
    "RigState",
    "RigctldError",
    "FlexRadioService",
    "FlexState",
    "FlexRadioError",
]



"""Main logging screen."""

import logging
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.events import Focus
from textual.screen import Screen
from textual.timer import Timer
from textual.widgets import Footer, Input, Static
from textual.worker import Worker, WorkerState

from ..adif import export_adif_file, export_pota_adif, get_pota_filename, parse_adif_file
from ..callsign import LookupError
from ..config import DXClusterSource, RigControlType
from ..database import Database
from ..models import CallsignLookupResult, Spot
from ..modes import (
    ContestMode,
    FieldDayMode,
    ModeType,
    OperatingMode,
    POTAMode,
)
from ..services import (
    DXClusterService,
    FlexRadioService,
    FlexState,
    Park,
    POTAParksService,
    POTASpotService,
    RigctldService,
    RigState,
)
from ..widgets.qso_entry import QSOEntryForm
from ..widgets.qso_table import QSOTable
from ..widgets.spots_table import SpotsTable
from .file_picker import ExportCompleteScreen, FilePickerScreen
from .log_browser import LogBrowserScreen
from .mode_setup import (
    ContestSetupScreen,
    FieldDaySetupScreen,
    ModeSelectScreen,
    POTAHunterSetupScreen,
    POTASetupScreen,
)
from .help import HelpScreen
from .log_manager import LogManagerScreen
from .settings import SettingsScreen

logger = logging.getLogger(__name__)


class CallsignInfo(Static):
    """Widget to display callsign lookup information."""

    DEFAULT_CSS = """
    CallsignInfo {
        height: 1;
        padding: 0 1;
        background: $surface;
    }
    """

    def __init__(self, id: Optional[str] = None) -> None:
        super().__init__(id=id)
        self._info = ""

    def set_info(self, info: str) -> None:
        """Set the callsign info text."""
        self._info = info
        self.update(f"[bold]Callsign Info:[/bold] {info}")

    def clear(self) -> None:
        """Clear the callsign info."""
        self._info = ""
        self.update("[dim]Callsign lookup: Enter a callsign to search...[/dim]")


class BandIndicator(Static):
    """Widget to display current band/mode/frequency."""

    DEFAULT_CSS = """
    BandIndicator {
        width: 100%;
        height: 1;
        padding: 0 1;
        background: $surface;
    }
    """

    def set_band_info(self, freq: float, mode: str, band: str) -> None:
        """Set the band indicator display."""
        self.update(f"[bold cyan]{freq:.3f} MHz[/bold cyan] | [green]{mode}[/green] | [yellow]{band}[/yellow]")


class UTCClock(Static):
    """Widget to display current UTC time."""

    DEFAULT_CSS = """
    UTCClock {
        width: auto;
        height: 1;
        padding: 0 1;
        text-align: right;
        color: $text;
        text-style: bold;
    }
    """

    def on_mount(self) -> None:
        """Start the clock update interval."""
        self._update_time()
        self.set_interval(1.0, self._update_time)

    def _update_time(self) -> None:
        """Update the displayed time."""
        now = datetime.now(timezone.utc)
        self.update(f"[yellow]UTC[/yellow] [bold white]{now.strftime('%H:%M:%S')}[/bold white] [dim]{now.strftime('%Y-%m-%d')}[/dim]")


class AppHeader(Static):
    """Custom application header with title and UTC clock."""

    DEFAULT_CSS = """
    AppHeader {
        width: 100%;
        height: 1;
        background: $primary;
        color: $text;
        layout: horizontal;
    }

    AppHeader .header-title {
        width: 1fr;
        height: 1;
        padding: 0 1;
        text-style: bold;
    }

    AppHeader .header-clock {
        width: auto;
        height: 1;
        padding: 0 1;
    }
    """

    def compose(self) -> ComposeResult:
        yield Static("[bold]TermLogger[/bold] - Amateur Radio Logger", classes="header-title")
        yield Static("", id="header-clock", classes="header-clock")

    def on_mount(self) -> None:
        """Start the clock update interval."""
        self._update_time()
        self.set_interval(1.0, self._update_time)

    def _update_time(self) -> None:
        """Update the displayed time."""
        now = datetime.now(timezone.utc)
        clock = self.query_one("#header-clock", Static)
        clock.update(f"[yellow]UTC {now.strftime('%H:%M:%S')}[/yellow] {now.strftime('%Y-%m-%d')}")


class StatusBar(Static):
    """Widget for status bar at bottom."""

    DEFAULT_CSS = """
    StatusBar {
        height: 1;
        dock: bottom;
        background: $primary;
        color: $text;
        padding: 0 1;
    }
    """

    def set_qso_count(self, count: int) -> None:
        """Set the QSO count display."""
        self.update(f"QSOs: {count}")


class ModeStatus(Static):
    """Widget to display current operating mode status."""

    DEFAULT_CSS = """
    ModeStatus {
        height: 1;
        padding: 0 1;
        background: $surface-darken-1;
    }
    """

    def __init__(self, id: Optional[str] = None) -> None:
        super().__init__(id=id)
        self._mode: Optional[OperatingMode] = None

    def set_mode(self, mode: Optional[OperatingMode]) -> None:
        """Set the current operating mode."""
        self._mode = mode
        self._update_display()

    def _update_display(self) -> None:
        """Update the mode status display."""
        if self._mode is None:
            self.update("[dim]Mode: General Logging | Ctrl+N to start a contest/activation[/dim]")
        else:
            status = self._mode.get_status_text()
            self.update(f"[bold cyan]{status}[/bold cyan]")

    def refresh_status(self) -> None:
        """Refresh the status display."""
        self._update_display()


class LogStatus(Static):
    """Widget to display current active log."""

    DEFAULT_CSS = """
    LogStatus {
        height: 1;
        padding: 0 1;
        background: $surface;
    }
    """

    def __init__(self, id: Optional[str] = None) -> None:
        super().__init__(id=id)
        self._log_name: Optional[str] = None
        self._qso_count: int = 0

    def set_log(self, name: Optional[str], qso_count: int = 0) -> None:
        """Set the current log display."""
        self._log_name = name
        self._qso_count = qso_count
        self._update_display()

    def _update_display(self) -> None:
        """Update the log status display."""
        if self._log_name is None:
            self.update("[dim]Log: All QSOs | F6 to manage logs[/dim]")
        else:
            self.update(f"[bold green]Log:[/bold green] {self._log_name} [dim]({self._qso_count} QSOs)[/dim] | [dim]F6 to change[/dim]")


class MainScreen(Screen):
    """Main logging screen."""

    BINDINGS = [
        ("f1", "show_help", "Help"),
        ("f2", "export_adif", "Export"),
        ("f3", "clear_form", "Clear"),
        ("f4", "show_settings", "Settings"),
        ("f5", "lookup_callsign", "Lookup"),
        ("f6", "manage_logs", "Logs"),
        ("f7", "browse_log", "Browse"),
        ("f8", "export_cabrillo", "Cabrillo"),
        ("f9", "end_mode", "End Mode"),
        ("f10", "quit", "Exit"),
        ("ctrl+n", "new_contest", "New Contest"),
        ("ctrl+e", "export_adif", "Export ADIF"),
        ("ctrl+i", "import_adif", "Import ADIF"),
        ("ctrl+p", "export_pota", "Export POTA"),
    ]

    CSS = """
    MainScreen {
        layout: vertical;
    }

    .main-container {
        height: 1fr;
        padding: 0 1;
    }

    #tables-container {
        height: 1fr;
        min-height: 10;
        width: 100%;
    }

    #spots-table {
        width: 2fr;
    }

    #qso-table {
        width: 3fr;
    }
    """

    def __init__(self, db: Database) -> None:
        super().__init__()
        self.db = db
        self._current_mode: Optional[OperatingMode] = None
        self._active_log_id: Optional[int] = None
        self._spot_timer: Optional[Timer] = None
        self._pota_spot_service: Optional[POTASpotService] = None
        self._pota_parks_service: Optional[POTAParksService] = None
        self._dx_cluster_service: Optional[DXClusterService] = None
        self._rigctld_service: Optional[RigctldService] = None
        self._flexradio_service: Optional[FlexRadioService] = None
        self._rig_poll_timer: Optional[Timer] = None

    def compose(self) -> ComposeResult:
        """Create child widgets."""
        yield AppHeader(id="app-header")

        with Vertical(classes="main-container"):
            # Band/mode indicator
            yield BandIndicator(id="band-indicator")

            # Current log status
            yield LogStatus(id="log-status")

            # Operating mode status
            yield ModeStatus(id="mode-status")

            # QSO entry form
            yield QSOEntryForm(id="qso-form")

            # Callsign lookup info
            yield CallsignInfo(id="callsign-info")

            # Tables container (Spots table + QSO table)
            with Horizontal(id="tables-container"):
                yield SpotsTable(id="spots-table", title="DX Spots")
                yield QSOTable(id="qso-table")

        yield StatusBar(id="status-bar")
        yield Footer()

    def on_mount(self) -> None:
        """Initialize screen when mounted."""
        # Load active log
        self._load_active_log()

        # Load recent QSOs (filtered by active log if set)
        self._refresh_qso_table()

        # Initialize callsign info
        self.query_one(CallsignInfo).clear()

        # Set initial band indicator
        self.query_one(BandIndicator).set_band_info(14.250, "SSB", "20m")

        # Initialize spot services
        self._pota_spot_service = POTASpotService()
        self._pota_parks_service = POTAParksService()
        self._dx_cluster_service = DXClusterService(
            host=self.app.config.dx_cluster_host,
            port=self.app.config.dx_cluster_port,
            callsign=self.app.config.dx_cluster_callsign or self.app.config.my_callsign,
        )

        # Start spot refresh based on current mode
        self._start_spot_refresh()

        # Initialize rig control service based on type
        rig_type = self.app.config.rig_control_type
        if rig_type == RigControlType.RIGCTLD:
            self._rigctld_service = RigctldService(
                host=self.app.config.rigctld_host,
                port=self.app.config.rigctld_port,
                on_state_change=self._on_rig_state_change,
            )
            self._start_rig_polling()
        elif rig_type == RigControlType.FLEXRADIO:
            self._flexradio_service = FlexRadioService(
                host=self.app.config.flexradio_host,
                port=self.app.config.flexradio_port,
                on_state_change=self._on_flex_state_change,
            )
            self._start_rig_polling()

    def _load_active_log(self) -> None:
        """Load the active log from database."""
        active_log = self.db.get_active_log()
        if active_log:
            self._active_log_id = active_log.id
            self.query_one(LogStatus).set_log(active_log.display_name, active_log.qso_count)
        else:
            self._active_log_id = None
            self.query_one(LogStatus).set_log(None)

    def _refresh_qso_table(self) -> None:
        """Refresh the QSO table with current log filter."""
        qsos = self.db.get_recent_qsos(50, log_id=self._active_log_id)
        self.query_one(QSOTable).load_qsos(qsos)

        # Update status bar with count
        count = self.db.get_qso_count(log_id=self._active_log_id)
        self.query_one(StatusBar).set_qso_count(count)

    # Rig control methods
    def _start_rig_polling(self) -> None:
        """Start polling the rig for frequency/mode changes."""
        self._stop_rig_polling()
        rig_type = self.app.config.rig_control_type
        if rig_type != RigControlType.NONE and (self._rigctld_service or self._flexradio_service):
            interval = self.app.config.rig_poll_interval
            self._rig_poll_timer = self.set_interval(interval, self._poll_rig)
            # Do initial poll
            self.run_worker(self._do_rig_poll(), exclusive=False, name="rig_poll")

    def _stop_rig_polling(self) -> None:
        """Stop rig polling."""
        if self._rig_poll_timer:
            self._rig_poll_timer.stop()
            self._rig_poll_timer = None

    def _poll_rig(self) -> None:
        """Trigger a rig poll."""
        self.run_worker(self._do_rig_poll(), exclusive=False, name="rig_poll")

    async def _do_rig_poll(self) -> None:
        """Async worker to poll rig state."""
        if self._rigctld_service:
            await self._rigctld_service.poll()
        elif self._flexradio_service:
            await self._flexradio_service.poll()

    def _on_rig_state_change(self, state: RigState) -> None:
        """Handle rigctld state change callback - update UI."""
        # Use call_later for thread-safe UI update
        self.app.call_later(self._update_rigctld_display, state)

    def _on_flex_state_change(self, state: FlexState) -> None:
        """Handle Flex Radio state change callback - update UI."""
        logger.debug(f"Flex state change: {state.frequency_mhz:.3f} MHz {state.mode}")
        # Use call_later for thread-safe UI update
        self.app.call_later(self._update_flex_display, state)

    def _update_rigctld_display(self, state: RigState) -> None:
        """Update UI with new rigctld state."""
        try:
            # Update band indicator only - don't overwrite QSO form
            # User-initiated actions (spot clicks) should control the form
            band_indicator = self.query_one(BandIndicator)
            band_str = state.band or "?"
            mapped_mode = RigctldService.map_mode_from_rigctld(state.mode)
            band_indicator.set_band_info(state.frequency_mhz, mapped_mode, band_str)
        except Exception as e:
            logger.warning(f"Failed to update rig display: {e}")

    def _update_flex_display(self, state: FlexState) -> None:
        """Update UI with new Flex Radio state."""
        try:
            # Update band indicator only - don't overwrite QSO form
            # User-initiated actions (spot clicks) should control the form
            band_indicator = self.query_one(BandIndicator)
            band_str = state.band or "?"
            mapped_mode = FlexRadioService.map_mode_from_flex(state.mode)
            logger.debug(f"Flex display update: {state.frequency_mhz:.3f} MHz {mapped_mode} ({band_str})")
            band_indicator.set_band_info(state.frequency_mhz, mapped_mode, band_str)
        except Exception as e:
            logger.warning(f"Failed to update Flex display: {e}")

    def on_focus(self, event: Focus) -> None:
        """Handle focus events - update frequency from rig when frequency field focused."""
        # Check if the focused widget is the frequency input
        # Try multiple ways to get the focused widget ID
        widget_id = None
        if hasattr(event, "widget"):
            widget_id = getattr(event.widget, "id", None)
        elif hasattr(event, "control"):
            widget_id = getattr(event.control, "id", None)

        if widget_id == "frequency":
            self._update_frequency_from_rig()

    def _update_frequency_from_rig(self) -> None:
        """Update the frequency field from the current rig state."""
        rig_type = self.app.config.rig_control_type
        freq_mhz = 0.0
        mode = None

        if rig_type == RigControlType.RIGCTLD and self._rigctld_service:
            state = self._rigctld_service.last_state
            if state:
                freq_mhz = state.frequency_mhz
                mode = RigctldService.map_mode_from_rigctld(state.mode)
        elif rig_type == RigControlType.FLEXRADIO and self._flexradio_service:
            state = self._flexradio_service.last_state
            if state and state.frequency > 0:
                freq_mhz = state.frequency_mhz
                mode = FlexRadioService.map_mode_from_flex(state.mode)

        if freq_mhz > 0:
            try:
                form = self.query_one(QSOEntryForm)
                form.set_frequency(freq_mhz)
                logger.debug(f"Updated frequency from rig: {freq_mhz:.3f} MHz")
                if mode:
                    try:
                        form.set_mode(mode)
                    except (ValueError, KeyError):
                        pass
            except Exception as e:
                logger.warning(f"Failed to update frequency from rig: {e}")

    async def on_unmount(self) -> None:
        """Clean up when screen is unmounted."""
        self._stop_spot_refresh()
        self._stop_rig_polling()
        if self._pota_spot_service:
            await self._pota_spot_service.close()
        if self._pota_parks_service:
            await self._pota_parks_service.close()
        if self._dx_cluster_service:
            await self._dx_cluster_service.close()
        if self._rigctld_service:
            await self._rigctld_service.close()
        if self._flexradio_service:
            await self._flexradio_service.close()

    def _start_spot_refresh(self) -> None:
        """Start the spot refresh timer based on current mode."""
        self._stop_spot_refresh()

        # Determine refresh interval based on mode
        spots_table = self.query_one(SpotsTable)
        if self._current_mode and self._current_mode.mode_type == ModeType.POTA:
            if self.app.config.pota_spots_enabled:
                interval = self.app.config.pota_spots_refresh_seconds
                self._spot_timer = self.set_interval(interval, self._refresh_pota_spots)
                spots_table.set_title("POTA Spots")
                spots_table.reset_filters()  # Reset filters when switching sources
                # Do initial fetch
                self.run_worker(self._fetch_pota_spots(), exclusive=True)
        else:
            if self.app.config.dx_cluster_enabled:
                interval = self.app.config.dx_cluster_refresh_seconds
                self._spot_timer = self.set_interval(interval, self._refresh_dx_spots)
                spots_table.set_title("DX Spots")
                spots_table.reset_filters()  # Reset filters when switching sources
                # Do initial fetch
                self.run_worker(self._fetch_dx_spots(), exclusive=True)

    def _stop_spot_refresh(self) -> None:
        """Stop the spot refresh timer."""
        if self._spot_timer:
            self._spot_timer.stop()
            self._spot_timer = None

    def _refresh_pota_spots(self) -> None:
        """Trigger POTA spots refresh."""
        self.run_worker(self._fetch_pota_spots(), exclusive=True)

    def _refresh_dx_spots(self) -> None:
        """Trigger DX cluster spots refresh."""
        self.run_worker(self._fetch_dx_spots(), exclusive=True)

    async def _fetch_pota_spots(self) -> list[Spot]:
        """Fetch POTA spots asynchronously."""
        try:
            if self._pota_spot_service:
                spots = await self._pota_spot_service.get_spots(limit=50)
                # Use call_later to safely update UI from async worker
                self.app.call_later(self._update_spots_table, spots)
                return spots
        except Exception as e:
            logger.error(f"Error fetching POTA spots: {e}")
            self.notify(f"POTA spots error: {e}", severity="warning", timeout=3)
        return []

    async def _fetch_dx_spots(self) -> list[Spot]:
        """Fetch DX cluster spots asynchronously."""
        try:
            if self._dx_cluster_service:
                source = self.app.config.dx_cluster_source
                use_telnet = source in (DXClusterSource.TELNET, DXClusterSource.BOTH)
                use_web = source in (DXClusterSource.WEB_API, DXClusterSource.BOTH)
                spots = await self._dx_cluster_service.get_spots(
                    limit=50,
                    use_telnet=use_telnet,
                    use_web=use_web,
                )
                # Use call_later to safely update UI from async worker
                self.app.call_later(self._update_spots_table, spots)
                return spots
        except Exception as e:
            logger.error(f"Error fetching DX spots: {e}")
            self.notify(f"DX spots error: {e}", severity="warning", timeout=3)
        return []

    def _update_spots_table(self, spots: list[Spot]) -> None:
        """Update the spots table with new spots."""
        try:
            spots_table = self.query_one(SpotsTable)
            spots_table.load_spots(spots)
            logger.info(f"Updated spots table with {len(spots)} spots")
        except Exception as e:
            logger.error(f"Failed to update spots table: {e}")

    def on_spots_table_spot_selected(self, event: SpotsTable.SpotSelected) -> None:
        """Handle spot selection - auto-fill the QSO form and optionally QSY radio."""
        spot = event.spot
        logger.info(f"Spot selected: {spot.callsign} {spot.frequency:.3f} MHz {spot.mode}")
        form = self.query_one(QSOEntryForm)

        # Set callsign
        try:
            callsign_input = self.query_one("#callsign", Input)
            callsign_input.value = spot.callsign
            logger.debug(f"Set callsign to {spot.callsign}")
        except Exception as e:
            logger.warning(f"Failed to set callsign: {e}")

        # Set frequency and mode
        try:
            form.set_frequency(spot.frequency)
            logger.debug(f"Set frequency to {spot.frequency}")
            if spot.mode:
                form.set_mode(spot.mode)
                logger.debug(f"Set mode to {spot.mode}")
        except Exception as e:
            logger.warning(f"Failed to set frequency/mode: {e}")

        # QSY radio if enabled
        rig_type = self.app.config.rig_control_type
        if rig_type != RigControlType.NONE and self.app.config.rig_auto_qsy:
            if (rig_type == RigControlType.RIGCTLD and self._rigctld_service) or (
                rig_type == RigControlType.FLEXRADIO and self._flexradio_service
            ):
                self.run_worker(
                    self._qsy_to_spot(spot),
                    exclusive=False,
                    name="qsy",
                )

        # For POTA spots, look up park info and display, and fill in park ref
        if spot.park_reference:
            # Fill in their park reference in the form (for P2P contacts)
            form.set_their_park(spot.park_reference)
            # Look up park info to display
            self.run_worker(
                self._lookup_park(spot.park_reference),
                exclusive=False,
                name=f"park_{spot.park_reference}",
            )
        elif spot.callsign:
            # For DX spots, do regular callsign lookup
            self._do_lookup(spot.callsign)

        self.notify(f"Selected {spot.callsign} on {spot.frequency:.3f} MHz", timeout=2)

    async def _lookup_park(self, park_reference: str) -> Optional[Park]:
        """Look up POTA park information."""
        if not self._pota_parks_service:
            return None

        try:
            park = await self._pota_parks_service.get_park(park_reference)
            if park:
                # Update callsign info with park details
                self.app.call_later(self._display_park_info, park)
            return park
        except Exception as e:
            logger.warning(f"Park lookup failed: {e}")
            return None

    def _display_park_info(self, park: Park) -> None:
        """Display park information in the callsign info area."""
        try:
            callsign_info = self.query_one(CallsignInfo)
            info_parts = [f"[bold cyan]{park.reference}[/bold cyan]: {park.name}"]
            if park.parktype:
                info_parts.append(f"[dim]({park.parktype})[/dim]")
            if park.location_name:
                info_parts.append(f"[green]{park.location_name}[/green]")
            if park.grid6:
                info_parts.append(f"[yellow]{park.grid6}[/yellow]")
            callsign_info.set_info(" | ".join(info_parts))
        except Exception as e:
            logger.warning(f"Failed to display park info: {e}")

    async def _qsy_to_spot(self, spot: Spot) -> None:
        """QSY the radio to a spot's frequency and mode."""
        try:
            rig_type = self.app.config.rig_control_type
            if rig_type == RigControlType.RIGCTLD and self._rigctld_service:
                await self._rigctld_service.set_frequency_mhz(spot.frequency)
                if spot.mode:
                    rig_mode = RigctldService.map_mode_to_rigctld(
                        spot.mode, spot.frequency
                    )
                    await self._rigctld_service.set_mode(rig_mode)
                logger.info(f"QSY to {spot.frequency:.3f} MHz {spot.mode or ''}")
            elif rig_type == RigControlType.FLEXRADIO and self._flexradio_service:
                await self._flexradio_service.set_frequency_mhz(spot.frequency)
                if spot.mode:
                    flex_mode = FlexRadioService.map_mode_to_flex(
                        spot.mode, spot.frequency
                    )
                    await self._flexradio_service.set_mode(flex_mode)
                logger.info(f"QSY to {spot.frequency:.3f} MHz {spot.mode or ''}")
        except Exception as e:
            logger.error(f"QSY failed: {e}")
            self.app.call_later(
                self.notify,
                f"QSY failed: {e}",
                severity="warning",
            )

    def on_qso_entry_form_qso_logged(self, event: QSOEntryForm.QSOLogged) -> None:
        """Handle QSO logged event."""
        # If in a mode, set exchange sent
        if self._current_mode:
            event.qso.exchange_sent = self._current_mode.format_exchange_sent()

        # Save to database with active log
        qso_id = self.db.add_qso(event.qso, log_id=self._active_log_id)
        event.qso.id = qso_id
        event.qso.log_id = self._active_log_id

        # Add to current mode if active
        if self._current_mode:
            self._current_mode.add_qso(event.qso)
            self.query_one(ModeStatus).refresh_status()

        # Add to table
        self.query_one(QSOTable).add_qso(event.qso)

        # Update status bar with log-filtered count
        count = self.db.get_qso_count(log_id=self._active_log_id)
        self.query_one(StatusBar).set_qso_count(count)

        # Update log status with new count
        if self._active_log_id:
            active_log = self.db.get_active_log()
            if active_log:
                self.query_one(LogStatus).set_log(active_log.display_name, active_log.qso_count)

        # Clear callsign info
        self.query_one(CallsignInfo).clear()

        self.notify(f"Logged {event.qso.callsign}", timeout=2)

    def on_qso_entry_form_callsign_changed(
        self, event: QSOEntryForm.CallsignChanged
    ) -> None:
        """Handle callsign change for dupe checking and lookup."""
        # Check for dupes
        is_dupe = self.db.check_dupe(event.callsign)
        self.query_one(QSOEntryForm).set_dupe_status(is_dupe)

        # Trigger callsign lookup if auto-lookup is enabled
        if len(event.callsign) >= 3 and self.app.config.auto_lookup:
            self._do_lookup(event.callsign)

    def on_qso_entry_form_callsign_blurred(
        self, event: QSOEntryForm.CallsignBlurred
    ) -> None:
        """Handle callsign field blur - trigger lookup when tabbing out."""
        if event.callsign:
            self._do_lookup(event.callsign)

    def _do_lookup(self, callsign: str) -> None:
        """Perform async callsign lookup."""
        callsign_info = self.query_one(CallsignInfo)
        callsign_info.set_info(f"Looking up {callsign}...")

        # Cancel any pending lookup
        self._cancel_lookup()

        # Start new lookup worker
        self._lookup_worker = self.run_worker(
            self._lookup_callsign_async(callsign),
            name=f"lookup_{callsign}",
            exclusive=True,
        )

    async def _lookup_callsign_async(self, callsign: str) -> Optional[CallsignLookupResult]:
        """Async worker to perform callsign lookup."""
        try:
            result = await self.app.lookup_service.lookup(callsign)
            return result
        except LookupError:
            return None

    def on_worker_state_changed(self, event: Worker.StateChanged) -> None:
        """Handle lookup worker completion."""
        if event.worker.name and event.worker.name.startswith("lookup_"):
            if event.state == WorkerState.SUCCESS:
                result = event.worker.result
                callsign_info = self.query_one(CallsignInfo)
                if result:
                    callsign_info.set_info(result.display_str)
                else:
                    callsign_info.set_info("No information found")
            elif event.state == WorkerState.ERROR:
                self.query_one(CallsignInfo).set_info("Lookup failed")

    def _cancel_lookup(self) -> None:
        """Cancel any pending lookup."""
        if hasattr(self, "_lookup_worker") and self._lookup_worker:
            self._lookup_worker.cancel()

    def action_show_help(self) -> None:
        """Show help screen."""
        self.app.push_screen(HelpScreen())

    def action_clear_form(self) -> None:
        """Clear the QSO entry form."""
        self.query_one(QSOEntryForm).clear_form()
        self.query_one(CallsignInfo).clear()

    def action_lookup_callsign(self) -> None:
        """Trigger manual callsign lookup."""
        # Get current callsign from form
        try:
            callsign_input = self.query_one("#callsign", Input)
            callsign = callsign_input.value.strip().upper()
            if callsign:
                self._do_lookup(callsign)
            else:
                self.notify("Enter a callsign first", severity="warning")
        except Exception:
            self.notify("Enter a callsign first", severity="warning")

    def action_show_settings(self) -> None:
        """Show settings screen."""
        self.app.push_screen(SettingsScreen(self.app.config))

    def action_manage_logs(self) -> None:
        """Show log manager screen."""

        def on_log_manager_close(log_id: Optional[int]) -> None:
            # Refresh the active log and QSO table
            self._load_active_log()
            self._refresh_qso_table()

        self.app.push_screen(LogManagerScreen(self.db), on_log_manager_close)

    def action_browse_log(self) -> None:
        """Show log browser screen."""

        def on_browser_close() -> None:
            # Refresh the table when returning from browser (respecting log filter)
            self._refresh_qso_table()

        self.app.push_screen(LogBrowserScreen(self.db), on_browser_close)

    def action_export_adif(self) -> None:
        """Export log to ADIF."""
        # Generate default filename with timestamp
        timestamp = datetime.now(timezone.utc).strftime("%Y%m%d_%H%M%S")
        default_filename = f"termlogger_export_{timestamp}.adi"

        def handle_export(path: Optional[Path]) -> None:
            if path is None:
                return

            try:
                # Get all QSOs from database
                qsos = self.db.get_all_qsos(limit=10000)
                count = export_adif_file(qsos, path)
                self.app.push_screen(
                    ExportCompleteScreen(f"Exported {count} QSOs to:\n{path}")
                )
            except Exception as e:
                self.notify(f"Export failed: {e}", severity="error")

        self.app.push_screen(
            FilePickerScreen(
                title="Export ADIF",
                start_path=Path.home(),
                extensions=[".adi", ".adif"],
                save_mode=True,
                default_filename=default_filename,
            ),
            handle_export,
        )

    def action_import_adif(self) -> None:
        """Import ADIF file."""

        def handle_import(path: Optional[Path]) -> None:
            if path is None:
                return

            try:
                qsos = parse_adif_file(path)
                imported_count = 0

                for qso in qsos:
                    self.db.add_qso(qso)
                    imported_count += 1

                # Refresh the table
                recent_qsos = self.db.get_recent_qsos(50)
                self.query_one(QSOTable).load_qsos(recent_qsos)

                # Update status
                count = self.db.get_qso_count()
                self.query_one(StatusBar).set_qso_count(count)

                self.app.push_screen(
                    ExportCompleteScreen(f"Imported {imported_count} QSOs from:\n{path}")
                )
            except FileNotFoundError:
                self.notify("File not found", severity="error")
            except Exception as e:
                self.notify(f"Import failed: {e}", severity="error")

        self.app.push_screen(
            FilePickerScreen(
                title="Import ADIF",
                start_path=Path.home(),
                extensions=[".adi", ".adif"],
                save_mode=False,
            ),
            handle_import,
        )

    def action_new_contest(self) -> None:
        """Start a new contest or operating mode."""

        def handle_mode_select(mode_type: Optional[ModeType]) -> None:
            if mode_type is None:
                return

            if mode_type == ModeType.GENERAL:
                # Clear current mode, return to general logging
                self._current_mode = None
                self.query_one(ModeStatus).set_mode(None)
                self.notify("Switched to General Logging")
            elif mode_type == ModeType.CONTEST:
                # Show contest setup
                self.app.push_screen(
                    ContestSetupScreen(
                        my_callsign=self.app.config.my_callsign,
                        my_exchange=self.app.config.my_cq_zone or "",
                    ),
                    handle_contest_setup,
                )
            elif mode_type == ModeType.POTA:
                # Show POTA activation setup
                self.app.push_screen(
                    POTASetupScreen(
                        my_callsign=self.app.config.my_callsign,
                        my_state=self.app.config.my_state or "",
                        my_grid=self.app.config.my_grid or "",
                    ),
                    handle_pota_setup,
                )
            elif mode_type == "pota_hunter":
                # Show POTA hunter setup
                self.app.push_screen(
                    POTAHunterSetupScreen(
                        my_callsign=self.app.config.my_callsign,
                        my_state=self.app.config.my_state or "",
                        my_grid=self.app.config.my_grid or "",
                    ),
                    handle_pota_hunter_setup,
                )
            elif mode_type == ModeType.FIELDDAY:
                # Show Field Day setup
                self.app.push_screen(
                    FieldDaySetupScreen(my_callsign=self.app.config.my_callsign),
                    handle_fieldday_setup,
                )

        def handle_contest_setup(mode: Optional[ContestMode]) -> None:
            if mode:
                self._current_mode = mode
                self.query_one(ModeStatus).set_mode(mode)
                self.notify(f"Started {mode.config.contest_name}")

        def handle_pota_setup(mode: Optional[POTAMode]) -> None:
            if mode:
                self._current_mode = mode
                self.query_one(ModeStatus).set_mode(mode)
                parks = ", ".join(mode.get_all_parks())
                self.notify(f"Started POTA activation: {parks}")
                # Enable POTA fields in QSO form
                self.query_one(QSOEntryForm).set_pota_mode(True)
                # Switch to POTA spots
                self._start_spot_refresh()

        def handle_pota_hunter_setup(mode: Optional[POTAMode]) -> None:
            if mode:
                self._current_mode = mode
                self.query_one(ModeStatus).set_mode(mode)
                self.notify("Started POTA hunting mode")
                # Enable POTA fields in QSO form
                self.query_one(QSOEntryForm).set_pota_mode(True)
                # Switch to POTA spots (hunters want to see activators)
                self._start_spot_refresh()

        def handle_fieldday_setup(mode: Optional[FieldDayMode]) -> None:
            if mode:
                self._current_mode = mode
                self.query_one(ModeStatus).set_mode(mode)
                self.notify(f"Started Field Day: {mode.config.my_class} {mode.config.my_section}")

        self.app.push_screen(ModeSelectScreen(), handle_mode_select)

    def action_end_mode(self) -> None:
        """End the current operating mode."""
        if self._current_mode is None:
            self.notify("No active mode to end", severity="warning")
            return

        mode_name = self._current_mode.name or "Mode"
        score = self._current_mode.calculate_score()
        # Check if it was a POTA mode before clearing
        was_pota_mode = isinstance(self._current_mode, POTAMode)
        self._current_mode = None
        self.query_one(ModeStatus).set_mode(None)
        self.notify(f"Ended {mode_name} - Final score: {score.total_score}")
        # Disable POTA fields in QSO form if we were in POTA mode
        if was_pota_mode:
            self.query_one(QSOEntryForm).set_pota_mode(False)
        # Switch back to DX cluster spots
        self._start_spot_refresh()

    def action_export_cabrillo(self) -> None:
        """Export log in Cabrillo format."""
        if self._current_mode is None:
            self.notify("No active contest mode for Cabrillo export", severity="warning")
            return

        # Generate default filename with timestamp
        timestamp = datetime.now(timezone.utc).strftime("%Y%m%d_%H%M%S")
        mode_name = self._current_mode.mode_type.value
        default_filename = f"termlogger_{mode_name}_{timestamp}.log"

        def handle_export(path: Optional[Path]) -> None:
            if path is None or self._current_mode is None:
                return

            try:
                cabrillo_content = self._current_mode.export_cabrillo()
                path.write_text(cabrillo_content)
                score = self._current_mode.calculate_score()
                self.app.push_screen(
                    ExportCompleteScreen(
                        f"Exported Cabrillo log to:\n{path}\n\n"
                        f"QSOs: {score.qso_count}\n"
                        f"Score: {score.total_score}"
                    )
                )
            except Exception as e:
                self.notify(f"Export failed: {e}", severity="error")

        self.app.push_screen(
            FilePickerScreen(
                title="Export Cabrillo",
                start_path=Path.home(),
                extensions=[".log", ".cbr"],
                save_mode=True,
                default_filename=default_filename,
            ),
            handle_export,
        )

    def action_export_pota(self) -> None:
        """Export log in POTA format for upload to pota.app."""
        # Check if in POTA mode
        if not isinstance(self._current_mode, POTAMode):
            self.notify("Start a POTA activation first (Ctrl+N)", severity="warning")
            return

        pota_mode = self._current_mode
        if not pota_mode.config.my_park:
            self.notify("No park reference configured", severity="warning")
            return

        # Get QSOs from this activation
        qsos = pota_mode._qsos
        if not qsos:
            self.notify("No QSOs to export", severity="warning")
            return

        # Generate POTA filename
        default_filename = get_pota_filename(
            pota_mode.config.my_callsign,
            pota_mode.config.my_park,
        )

        def handle_export(path: Optional[Path]) -> None:
            if path is None:
                return

            try:
                count = export_pota_adif(
                    qsos,
                    path,
                    my_callsign=pota_mode.config.my_callsign,
                    my_park_ref=pota_mode.config.my_park,
                    my_state=pota_mode.config.my_state,
                    my_grid=pota_mode.config.my_grid,
                )
                score = pota_mode.calculate_score()
                self.app.push_screen(
                    ExportCompleteScreen(
                        f"Exported {count} QSOs for POTA to:\n{path}\n\n"
                        f"Park: {pota_mode.config.my_park}\n"
                        f"P2P Contacts: {score.multipliers}\n\n"
                        f"Upload at: pota.app → My Log Uploads"
                    )
                )
            except Exception as e:
                self.notify(f"POTA export failed: {e}", severity="error")

        self.app.push_screen(
            FilePickerScreen(
                title="Export for POTA",
                start_path=Path.home(),
                extensions=[".adi", ".adif"],
                save_mode=True,
                default_filename=default_filename,
            ),
            handle_export,
        )

    def action_quit(self) -> None:
        """Quit the application."""
        self.app.exit()



"""ARRL Field Day mode implementation."""

from dataclasses import dataclass, field
from typing import Optional

from .base import ModeConfig, ModeScore, ModeType, OperatingMode


# ARRL/RAC sections
ARRL_SECTIONS = [
    # US Call Areas
    "CT", "EMA", "ME", "NH", "RI", "VT", "WMA",  # 1
    "ENY", "NLI", "NNJ", "NNY", "SNJ", "WNY",  # 2
    "DE", "EPA", "MDC", "WPA",  # 3
    "AL", "GA", "KY", "NC", "NFL", "SC", "SFL", "WCF", "TN", "VA", "PR", "VI",  # 4
    "AR", "LA", "MS", "NM", "NTX", "OK", "STX", "WTX",  # 5
    "EB", "LAX", "ORG", "SB", "SCV", "SDG", "SF", "SJV", "SV", "PAC",  # 6
    "AK", "AZ", "EWA", "ID", "MT", "NV", "OR", "UT", "WWA", "WY",  # 7
    "MI", "OH", "WV",  # 8
    "IL", "IN", "WI",  # 9
    "CO", "IA", "KS", "MN", "MO", "NE", "ND", "SD",  # 0
    # Canada
    "AB", "BC", "GH", "MB", "NB", "NL", "NS", "NT", "NU", "ON", "PE", "QC", "SK", "YT",
    # DX
    "DX",
]

# Valid Field Day classes
FD_CLASSES = ["1A", "1B", "1C", "1D", "1E", "2A", "2B", "2C", "2D", "2E",
              "3A", "3B", "3C", "3D", "3E", "4A", "4B", "4C", "4D", "4E",
              "5A", "5B", "5C", "5D", "5E", "6A", "6B", "6C", "6D", "6E",
              "7A", "8A", "9A", "10A", "11A", "12A", "13A", "14A", "15A",
              "1AB", "2AB", "3AB", "4AB", "5AB",  # Battery classes
              "1F", "2F", "3F", "4F", "5F"]  # Free VHF


@dataclass
class FieldDayConfig(ModeConfig):
    """Configuration for Field Day mode."""

    # Station info
    my_callsign: str = ""
    my_class: str = ""  # e.g., "3A"
    my_section: str = ""  # e.g., "SNJ"
    my_gota_callsign: str = ""  # Get On The Air station

    # Club info
    club_name: str = ""

    # Power levels
    power_level: str = "HIGH"  # HIGH (>150W), LOW (<=150W), QRP (<=5W)

    # Band/mode restrictions
    bands_used: list[str] = field(default_factory=list)

    # Bonus points tracking
    emergency_power: bool = False  # 100% emergency power
    media_publicity: bool = False
    public_location: bool = False
    public_info_table: bool = False
    message_to_sm: bool = False  # Message to Section Manager
    message_from_sm: bool = False
    w1aw_bulletin: bool = False
    educational_activity: bool = False
    elected_official: bool = False
    agency_representative: bool = False
    gota_bonus: bool = False
    web_submission: bool = False
    youth_participation: bool = False
    social_media: bool = False
    safety_officer: bool = False


class FieldDayMode(OperatingMode):
    """ARRL Field Day operating mode."""

    mode_type = ModeType.FIELDDAY

    # Point values
    CW_DIGITAL_POINTS = 2
    PHONE_POINTS = 1

    def __init__(self, config: FieldDayConfig):
        super().__init__(config)
        self.config: FieldDayConfig = config
        self._sections_worked: dict[str, set[str]] = {}  # band -> sections

    def get_exchange_fields(self) -> list[dict]:
        """Get Field Day exchange fields."""
        return [
            {"id": "class", "label": "Class", "width": 5, "required": True, "hint": "e.g., 3A"},
            {"id": "section", "label": "Section", "width": 6, "required": True, "hint": "e.g., SNJ"},
        ]

    def validate_exchange(self, exchange: dict) -> tuple[bool, str]:
        """Validate Field Day exchange."""
        fd_class = exchange.get("class", "").strip().upper()
        section = exchange.get("section", "").strip().upper()

        if not fd_class:
            return False, "Class is required"

        if not section:
            return False, "Section is required"

        # Validate class format (number + letter(s))
        if not any(fd_class.startswith(str(i)) for i in range(1, 30)):
            return False, "Invalid class format (e.g., 3A, 1E)"

        # Validate section
        if section not in ARRL_SECTIONS:
            return False, f"Unknown section: {section}"

        return True, ""

    def calculate_score(self) -> ModeScore:
        """Calculate Field Day score."""
        score = ModeScore()
        score.qso_count = len(self._qsos)

        cw_digital_modes = {"CW", "RTTY", "PSK31", "FT8", "FT4", "JS8", "DIGITAL"}

        for qso in self._qsos:
            # Point calculation
            if qso.mode.value.upper() in cw_digital_modes:
                score.qso_points += self.CW_DIGITAL_POINTS
            else:
                score.qso_points += self.PHONE_POINTS

            # Track sections per band
            band_key = qso.band.value if qso.band else "Unknown"
            if band_key not in self._sections_worked:
                self._sections_worked[band_key] = set()

            if qso.exchange_received:
                # Extract section from exchange (e.g., "3A SNJ" -> "SNJ")
                parts = qso.exchange_received.upper().split()
                if len(parts) >= 2:
                    section = parts[-1]
                    if section in ARRL_SECTIONS:
                        self._sections_worked[band_key].add(section)

            # Band breakdown
            score.band_breakdown[band_key] = score.band_breakdown.get(band_key, 0) + 1

            # Mode breakdown
            mode_key = qso.mode.value
            score.mode_breakdown[mode_key] = score.mode_breakdown.get(mode_key, 0) + 1

        # Count unique sections across all bands
        all_sections = set()
        for sections in self._sections_worked.values():
            all_sections.update(sections)
        score.multipliers = len(all_sections)

        # Base score
        base_score = score.qso_points

        # Power multiplier
        power_mult = self._get_power_multiplier()
        base_score = int(base_score * power_mult)

        # Add bonus points
        bonus = self._calculate_bonus_points()

        score.total_score = base_score + bonus

        return score

    def _get_power_multiplier(self) -> float:
        """Get power multiplier based on class and power level."""
        # Class A, B, F use transmitter count, E is emergency power
        # D is home station
        fd_class = self.config.my_class.upper()

        if "E" in fd_class:
            return 2.0  # Emergency power
        elif "B" in fd_class:
            return 2.0  # Battery
        elif self.config.power_level == "QRP":
            return 5.0  # QRP bonus
        elif self.config.power_level == "LOW":
            return 2.0  # Low power
        else:
            return 1.0  # High power

    def _calculate_bonus_points(self) -> int:
        """Calculate bonus points."""
        bonus = 0

        if self.config.emergency_power:
            bonus += 100  # 100% emergency power

        if self.config.media_publicity:
            bonus += 100

        if self.config.public_location:
            bonus += 100

        if self.config.public_info_table:
            bonus += 100

        if self.config.message_to_sm:
            bonus += 100

        if self.config.w1aw_bulletin:
            bonus += 100

        if self.config.educational_activity:
            bonus += 100

        if self.config.elected_official:
            bonus += 100

        if self.config.agency_representative:
            bonus += 100

        if self.config.gota_bonus:
            bonus += 100  # Up to 500 for GOTA

        if self.config.web_submission:
            bonus += 50

        if self.config.youth_participation:
            bonus += 100

        if self.config.social_media:
            bonus += 100

        if self.config.safety_officer:
            bonus += 100

        return bonus

    def check_dupe(self, callsign: str, band: Optional[str] = None, mode: Optional[str] = None) -> bool:
        """Check for duplicate contact.

        In Field Day, you can work the same station once per band per mode category.
        Mode categories: CW, Digital, Phone
        """
        # Normalize mode to category
        mode_category = self._get_mode_category(mode)
        dupe_key = f"{callsign.upper()}_{band}_{mode_category}"
        return dupe_key in self._dupes

    def _get_mode_category(self, mode: Optional[str]) -> str:
        """Get Field Day mode category."""
        if not mode:
            return "PHONE"

        mode_upper = mode.upper()
        if mode_upper == "CW":
            return "CW"
        elif mode_upper in {"RTTY", "PSK31", "FT8", "FT4", "JS8", "DIGITAL"}:
            return "DIGITAL"
        else:
            return "PHONE"

    def format_exchange_sent(self) -> str:
        """Format the exchange to send."""
        return f"{self.config.my_class} {self.config.my_section}"

    def get_next_serial(self) -> int:
        """Field Day doesn't use serial numbers."""
        return 0

    def export_cabrillo(self) -> str:
        """Export log in Cabrillo format."""
        score = self.calculate_score()

        lines = [
            "START-OF-LOG: 3.0",
            "CONTEST: ARRL-FIELD-DAY",
            f"CALLSIGN: {self.config.my_callsign}",
            f"LOCATION: {self.config.my_section}",
            f"ARRL-SECTION: {self.config.my_section}",
            f"CATEGORY: {self.config.my_class}",
            f"CATEGORY-POWER: {self.config.power_level}",
            f"CLUB: {self.config.club_name}",
            f"CLAIMED-SCORE: {score.total_score}",
            "CREATED-BY: TermLogger",
            f"SOAPBOX: QSO Points: {score.qso_points}",
            f"SOAPBOX: Sections: {score.multipliers}",
            f"SOAPBOX: Bonus: {self._calculate_bonus_points()}",
        ]

        # Add QSO records
        for qso in self._qsos:
            freq = int(qso.frequency * 1000)  # Convert to kHz
            mode = self._get_cabrillo_mode(qso.mode.value)
            date = qso.datetime_utc.strftime("%Y-%m-%d")
            time = qso.datetime_utc.strftime("%H%M")

            # Parse their exchange
            their_class = ""
            their_section = ""
            if qso.exchange_received:
                parts = qso.exchange_received.upper().split()
                if len(parts) >= 2:
                    their_class = parts[0]
                    their_section = parts[1]

            qso_line = (
                f"QSO: {freq:>5} {mode:>2} {date} {time} "
                f"{self.config.my_callsign:<13} {self.config.my_class:<4} {self.config.my_section:<4} "
                f"{qso.callsign:<13} {their_class:<4} {their_section:<4}"
            )
            lines.append(qso_line)

        lines.append("END-OF-LOG:")
        return "\n".join(lines)

    def _get_cabrillo_mode(self, mode: str) -> str:
        """Convert mode to Cabrillo format."""
        mode_upper = mode.upper()
        if mode_upper == "SSB":
            return "PH"
        elif mode_upper == "CW":
            return "CW"
        elif mode_upper in {"RTTY", "PSK31"}:
            return "RY"
        elif mode_upper in {"FT8", "FT4", "JS8", "DIGITAL"}:
            return "DG"
        else:
            return "PH"

    def get_status_text(self) -> str:
        """Get Field Day status text."""
        score = self.calculate_score()
        exchange = self.format_exchange_sent()

        return (
            f"Field Day {self.config.my_callsign} {exchange} | "
            f"QSOs: {score.qso_count} | "
            f"Pts: {score.qso_points} | "
            f"Sects: {score.multipliers} | "
            f"Score: {score.total_score}"
        )



"""ADIF (Amateur Data Interchange Format) import/export support.

ADIF is the standard format for exchanging amateur radio log data.
Format specification: https://adif.org/
"""

from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

from .models import Mode, QSO


# ADIF field name mappings
ADIF_FIELD_MAP = {
    "CALL": "callsign",
    "FREQ": "frequency",
    "MODE": "mode",
    "RST_SENT": "rst_sent",
    "RST_RCVD": "rst_received",
    "QSO_DATE": "date",
    "TIME_ON": "time",
    "COMMENT": "notes",
    "NOTES": "notes",
    "CONTEST_ID": "contest_id",
    "SRX": "exchange_received",
    "STX": "exchange_sent",
}

# Mode mappings from ADIF to our Mode enum
ADIF_MODE_MAP = {
    "SSB": Mode.SSB,
    "USB": Mode.SSB,
    "LSB": Mode.SSB,
    "CW": Mode.CW,
    "FM": Mode.FM,
    "AM": Mode.AM,
    "RTTY": Mode.RTTY,
    "PSK31": Mode.PSK31,
    "PSK": Mode.PSK31,
    "FT8": Mode.FT8,
    "FT4": Mode.FT4,
    "JS8": Mode.JS8,
    "SSTV": Mode.SSTV,
    "DIGI": Mode.DIGITAL,
    "DATA": Mode.DIGITAL,
    "MFSK": Mode.DIGITAL,
    "JT65": Mode.DIGITAL,
    "JT9": Mode.DIGITAL,
}


class ADIFParseError(Exception):
    """Error parsing ADIF data."""

    pass


def parse_adif_field(data: str, pos: int) -> tuple[Optional[tuple[str, str]], int]:
    """Parse a single ADIF field starting at position pos.

    Returns:
        Tuple of ((field_name, field_value), new_position) or (None, new_position)
    """
    # Skip whitespace
    while pos < len(data) and data[pos] in " \t\n\r":
        pos += 1

    if pos >= len(data):
        return None, pos

    # Look for field start
    if data[pos] != "<":
        # Skip non-field content
        while pos < len(data) and data[pos] != "<":
            pos += 1
        return None, pos

    # Find field end
    end_bracket = data.find(">", pos)
    if end_bracket == -1:
        return None, len(data)

    # Parse field header: <NAME:LENGTH> or <NAME:LENGTH:TYPE>
    field_header = data[pos + 1 : end_bracket]

    # Check for end of header or end of record
    if field_header.upper() in ("EOH", "EOR"):
        return (field_header.upper(), ""), end_bracket + 1

    # Parse field name and length
    parts = field_header.split(":")
    if len(parts) < 2:
        return None, end_bracket + 1

    field_name = parts[0].upper()
    try:
        field_length = int(parts[1])
    except ValueError:
        return None, end_bracket + 1

    # Extract field value
    value_start = end_bracket + 1
    value_end = value_start + field_length
    if value_end > len(data):
        value_end = len(data)

    field_value = data[value_start:value_end]

    return (field_name, field_value), value_end


def parse_adif(data: str) -> list[QSO]:
    """Parse ADIF data and return a list of QSO objects.

    Args:
        data: ADIF formatted string

    Returns:
        List of QSO objects

    Raises:
        ADIFParseError: If the data cannot be parsed
    """
    qsos = []
    pos = 0
    # Start assuming no header - if we see EOH, we know there was a header
    # Header-only fields that indicate we're in a header section
    header_fields = {"ADIF_VER", "PROGRAMID", "PROGRAMVERSION", "CREATED_TIMESTAMP"}
    in_header = False
    current_record: dict[str, str] = {}

    while pos < len(data):
        result, pos = parse_adif_field(data, pos)

        if result is None:
            continue

        field_name, field_value = result

        # Detect if we're in a header by seeing header-specific fields
        if field_name in header_fields:
            in_header = True
            continue

        if field_name == "EOH":
            # End of header - now in data section
            in_header = False
            continue

        if field_name == "EOR":
            # End of record - create QSO
            if current_record:
                try:
                    qso = _record_to_qso(current_record)
                    if qso:
                        qsos.append(qso)
                except Exception:
                    # Skip malformed records
                    pass
            current_record = {}
            continue

        # Only add to record if we're not in header
        if not in_header:
            current_record[field_name] = field_value

    return qsos


def _record_to_qso(record: dict[str, str]) -> Optional[QSO]:
    """Convert an ADIF record dictionary to a QSO object."""
    # Callsign is required
    callsign = record.get("CALL", "").strip().upper()
    if not callsign:
        return None

    # Parse frequency
    freq_str = record.get("FREQ", "14.0")
    try:
        frequency = float(freq_str)
    except ValueError:
        frequency = 14.0

    # Parse mode
    mode_str = record.get("MODE", "SSB").upper()
    mode = ADIF_MODE_MAP.get(mode_str, Mode.SSB)

    # Parse RST
    rst_sent = record.get("RST_SENT", "59")
    rst_received = record.get("RST_RCVD", "59")

    # Parse date and time
    date_str = record.get("QSO_DATE", "")
    time_str = record.get("TIME_ON", record.get("TIME_OFF", "0000"))

    # Normalize time string (could be HHMM or HHMMSS)
    time_str = time_str.ljust(6, "0")[:6]

    try:
        if date_str and len(date_str) >= 8:
            datetime_utc = datetime.strptime(
                f"{date_str[:8]}{time_str[:4]}", "%Y%m%d%H%M"
            )
        else:
            datetime_utc = datetime.now(timezone.utc).replace(tzinfo=None)
    except ValueError:
        datetime_utc = datetime.now(timezone.utc).replace(tzinfo=None)

    # Notes/comment
    notes = record.get("COMMENT", record.get("NOTES", ""))

    # Contest exchange
    exchange_sent = record.get("STX", record.get("STX_STRING", ""))
    exchange_received = record.get("SRX", record.get("SRX_STRING", ""))

    # Extended fields
    name = record.get("NAME", "")
    qth = record.get("QTH", "")
    state = record.get("STATE", "")
    country = record.get("COUNTRY", "")
    gridsquare = record.get("GRIDSQUARE", record.get("GRID", ""))

    # Parse numeric fields
    dxcc = None
    if "DXCC" in record:
        try:
            dxcc = int(record["DXCC"])
        except ValueError:
            pass

    cq_zone = None
    if "CQZ" in record:
        try:
            cq_zone = int(record["CQZ"])
        except ValueError:
            pass

    itu_zone = None
    if "ITUZ" in record:
        try:
            itu_zone = int(record["ITUZ"])
        except ValueError:
            pass

    tx_pwr = None
    if "TX_PWR" in record:
        try:
            tx_pwr = float(record["TX_PWR"])
        except ValueError:
            pass

    # Activity references
    pota_ref = record.get("POTA_REF", record.get("SIG_INFO", "") if record.get("SIG", "").upper() == "POTA" else "")
    sota_ref = record.get("SOTA_REF", record.get("SIG_INFO", "") if record.get("SIG", "").upper() == "SOTA" else "")
    iota = record.get("IOTA", "")
    sig = record.get("SIG", "")
    sig_info = record.get("SIG_INFO", "")

    # QSL fields
    qsl_sent = record.get("QSL_SENT", "")
    qsl_rcvd = record.get("QSL_RCVD", "")
    qsl_via = record.get("QSL_VIA", "")
    lotw_qsl_sent = record.get("LOTW_QSL_SENT", "")
    lotw_qsl_rcvd = record.get("LOTW_QSL_RCVD", "")

    # Propagation
    prop_mode = record.get("PROP_MODE", "")
    sat_name = record.get("SAT_NAME", "")

    # My station info
    my_gridsquare = record.get("MY_GRIDSQUARE", "")
    station_callsign = record.get("STATION_CALLSIGN", "")
    operator = record.get("OPERATOR", "")
    comment = record.get("COMMENT", record.get("NOTES", ""))

    return QSO(
        callsign=callsign,
        frequency=frequency,
        mode=mode,
        rst_sent=rst_sent,
        rst_received=rst_received,
        datetime_utc=datetime_utc,
        notes=notes,
        exchange_sent=exchange_sent if exchange_sent else None,
        exchange_received=exchange_received if exchange_received else None,
        # Extended fields
        name=name if name else None,
        qth=qth if qth else None,
        state=state if state else None,
        country=country if country else None,
        gridsquare=gridsquare if gridsquare else None,
        dxcc=dxcc,
        cq_zone=cq_zone,
        itu_zone=itu_zone,
        tx_pwr=tx_pwr,
        pota_ref=pota_ref if pota_ref else None,
        sota_ref=sota_ref if sota_ref else None,
        iota=iota if iota else None,
        sig=sig if sig else None,
        sig_info=sig_info if sig_info else None,
        qsl_sent=qsl_sent if qsl_sent else None,
        qsl_rcvd=qsl_rcvd if qsl_rcvd else None,
        qsl_via=qsl_via if qsl_via else None,
        lotw_qsl_sent=lotw_qsl_sent if lotw_qsl_sent else None,
        lotw_qsl_rcvd=lotw_qsl_rcvd if lotw_qsl_rcvd else None,
        prop_mode=prop_mode if prop_mode else None,
        sat_name=sat_name if sat_name else None,
        my_gridsquare=my_gridsquare if my_gridsquare else None,
        station_callsign=station_callsign if station_callsign else None,
        operator=operator if operator else None,
        comment=comment if comment else None,
    )


def parse_adif_file(filepath: str | Path) -> list[QSO]:
    """Parse an ADIF file and return a list of QSO objects.

    Args:
        filepath: Path to the ADIF file

    Returns:
        List of QSO objects

    Raises:
        FileNotFoundError: If the file doesn't exist
        ADIFParseError: If the file cannot be parsed
    """
    path = Path(filepath)
    if not path.exists():
        raise FileNotFoundError(f"ADIF file not found: {filepath}")

    with open(path, "r", encoding="utf-8", errors="replace") as f:
        data = f.read()

    return parse_adif(data)


def generate_adif_field(name: str, value: str) -> str:
    """Generate a single ADIF field.

    Args:
        name: Field name
        value: Field value

    Returns:
        Formatted ADIF field string
    """
    return f"<{name}:{len(value)}>{value}"


def generate_adif_header(
    program_name: str = "TermLogger",
    program_version: str = "0.1.0",
) -> str:
    """Generate an ADIF header.

    Args:
        program_name: Name of the program
        program_version: Version of the program

    Returns:
        ADIF header string
    """
    now = datetime.now(timezone.utc)
    lines = [
        "ADIF Export from TermLogger",
        f"Generated: {now.strftime('%Y-%m-%d %H:%M:%S')} UTC",
        "",
        generate_adif_field("ADIF_VER", "3.1.4"),
        generate_adif_field("PROGRAMID", program_name),
        generate_adif_field("PROGRAMVERSION", program_version),
        generate_adif_field("CREATED_TIMESTAMP", now.strftime("%Y%m%d %H%M%S")),
        "<EOH>",
        "",
    ]
    return "\n".join(lines)


def qso_to_adif(qso: QSO) -> str:
    """Convert a QSO to an ADIF record.

    Args:
        qso: QSO object to convert

    Returns:
        ADIF record string
    """
    fields = []

    # Required fields
    fields.append(generate_adif_field("CALL", qso.callsign.upper()))
    fields.append(generate_adif_field("FREQ", f"{qso.frequency:.6f}"))
    fields.append(generate_adif_field("MODE", qso.mode.value))

    # Band (derived from frequency)
    if qso.band:
        fields.append(generate_adif_field("BAND", qso.band.value))

    # RST
    fields.append(generate_adif_field("RST_SENT", qso.rst_sent))
    fields.append(generate_adif_field("RST_RCVD", qso.rst_received))

    # Date and time
    fields.append(generate_adif_field("QSO_DATE", qso.datetime_utc.strftime("%Y%m%d")))
    fields.append(generate_adif_field("TIME_ON", qso.datetime_utc.strftime("%H%M%S")))

    # Optional fields
    if qso.notes:
        fields.append(generate_adif_field("COMMENT", qso.notes))

    if qso.exchange_sent:
        fields.append(generate_adif_field("STX_STRING", qso.exchange_sent))

    if qso.exchange_received:
        fields.append(generate_adif_field("SRX_STRING", qso.exchange_received))

    # Extended fields - Station info
    if qso.name:
        fields.append(generate_adif_field("NAME", qso.name))
    if qso.qth:
        fields.append(generate_adif_field("QTH", qso.qth))
    if qso.state:
        fields.append(generate_adif_field("STATE", qso.state))
    if qso.country:
        fields.append(generate_adif_field("COUNTRY", qso.country))
    if qso.gridsquare:
        fields.append(generate_adif_field("GRIDSQUARE", qso.gridsquare))
    if qso.dxcc:
        fields.append(generate_adif_field("DXCC", str(qso.dxcc)))
    if qso.cq_zone:
        fields.append(generate_adif_field("CQZ", str(qso.cq_zone)))
    if qso.itu_zone:
        fields.append(generate_adif_field("ITUZ", str(qso.itu_zone)))
    if qso.continent:
        fields.append(generate_adif_field("CONT", qso.continent))

    # Power
    if qso.tx_pwr:
        fields.append(generate_adif_field("TX_PWR", str(qso.tx_pwr)))

    # Activity references
    if qso.pota_ref:
        fields.append(generate_adif_field("POTA_REF", qso.pota_ref))
    if qso.sota_ref:
        fields.append(generate_adif_field("SOTA_REF", qso.sota_ref))
    if qso.wwff_ref:
        fields.append(generate_adif_field("WWFF_REF", qso.wwff_ref))
    if qso.iota:
        fields.append(generate_adif_field("IOTA", qso.iota))
    if qso.sig:
        fields.append(generate_adif_field("SIG", qso.sig))
    if qso.sig_info:
        fields.append(generate_adif_field("SIG_INFO", qso.sig_info))

    # QSL fields
    if qso.qsl_sent:
        fields.append(generate_adif_field("QSL_SENT", qso.qsl_sent))
    if qso.qsl_rcvd:
        fields.append(generate_adif_field("QSL_RCVD", qso.qsl_rcvd))
    if qso.qsl_via:
        fields.append(generate_adif_field("QSL_VIA", qso.qsl_via))
    if qso.lotw_qsl_sent:
        fields.append(generate_adif_field("LOTW_QSL_SENT", qso.lotw_qsl_sent))
    if qso.lotw_qsl_rcvd:
        fields.append(generate_adif_field("LOTW_QSL_RCVD", qso.lotw_qsl_rcvd))
    if qso.eqsl_qsl_sent:
        fields.append(generate_adif_field("EQSL_QSL_SENT", qso.eqsl_qsl_sent))
    if qso.eqsl_qsl_rcvd:
        fields.append(generate_adif_field("EQSL_QSL_RCVD", qso.eqsl_qsl_rcvd))

    # Propagation
    if qso.prop_mode:
        fields.append(generate_adif_field("PROP_MODE", qso.prop_mode))
    if qso.sat_name:
        fields.append(generate_adif_field("SAT_NAME", qso.sat_name))
    if qso.sat_mode:
        fields.append(generate_adif_field("SAT_MODE", qso.sat_mode))

    # My station info
    if qso.station_callsign:
        fields.append(generate_adif_field("STATION_CALLSIGN", qso.station_callsign))
    if qso.operator:
        fields.append(generate_adif_field("OPERATOR", qso.operator))
    if qso.my_gridsquare:
        fields.append(generate_adif_field("MY_GRIDSQUARE", qso.my_gridsquare))
    if qso.my_pota_ref:
        fields.append(generate_adif_field("MY_POTA_REF", qso.my_pota_ref))
    if qso.my_sota_ref:
        fields.append(generate_adif_field("MY_SOTA_REF", qso.my_sota_ref))

    # End of record
    fields.append("<EOR>")

    return " ".join(fields)


def generate_adif(qsos: list[QSO], include_header: bool = True) -> str:
    """Generate ADIF data from a list of QSOs.

    Args:
        qsos: List of QSO objects
        include_header: Whether to include the ADIF header

    Returns:
        ADIF formatted string
    """
    lines = []

    if include_header:
        lines.append(generate_adif_header())

    for qso in qsos:
        lines.append(qso_to_adif(qso))
        lines.append("")  # Empty line between records for readability

    return "\n".join(lines)


def export_adif_file(
    qsos: list[QSO],
    filepath: str | Path,
    include_header: bool = True,
) -> int:
    """Export QSOs to an ADIF file.

    Args:
        qsos: List of QSO objects to export
        filepath: Path to the output file
        include_header: Whether to include the ADIF header

    Returns:
        Number of QSOs exported

    Raises:
        IOError: If the file cannot be written
    """
    path = Path(filepath)

    # Ensure .adi extension
    if path.suffix.lower() not in (".adi", ".adif"):
        path = path.with_suffix(".adi")

    adif_data = generate_adif(qsos, include_header)

    with open(path, "w", encoding="utf-8") as f:
        f.write(adif_data)

    return len(qsos)


def qso_to_pota_adif(
    qso: QSO,
    my_callsign: str,
    my_park_ref: str,
    my_state: str = "",
    my_grid: str = "",
) -> str:
    """Convert a QSO to a POTA-formatted ADIF record.

    This includes POTA-specific fields MY_SIG and MY_SIG_INFO.

    Args:
        qso: QSO object to convert
        my_callsign: Operator's callsign
        my_park_ref: Park reference being activated (e.g., "K-1234")
        my_state: Operator's state
        my_grid: Operator's grid square

    Returns:
        POTA-formatted ADIF record string
    """
    fields = []

    # Required POTA fields
    fields.append(generate_adif_field("CALL", qso.callsign.upper()))
    fields.append(generate_adif_field("QSO_DATE", qso.datetime_utc.strftime("%Y%m%d")))
    fields.append(generate_adif_field("TIME_ON", qso.datetime_utc.strftime("%H%M%S")))
    fields.append(generate_adif_field("MODE", qso.mode.value))

    # Band (required by POTA)
    if qso.band:
        fields.append(generate_adif_field("BAND", qso.band.value))

    # Frequency
    fields.append(generate_adif_field("FREQ", f"{qso.frequency:.6f}"))

    # RST
    fields.append(generate_adif_field("RST_SENT", qso.rst_sent))
    fields.append(generate_adif_field("RST_RCVD", qso.rst_received))

    # POTA-specific: My activation info
    fields.append(generate_adif_field("MY_SIG", "POTA"))
    fields.append(generate_adif_field("MY_SIG_INFO", my_park_ref.upper()))
    fields.append(generate_adif_field("STATION_CALLSIGN", my_callsign.upper()))
    fields.append(generate_adif_field("OPERATOR", my_callsign.upper()))

    if my_state:
        fields.append(generate_adif_field("MY_STATE", my_state.upper()))
    if my_grid:
        fields.append(generate_adif_field("MY_GRIDSQUARE", my_grid.upper()))

    # Park-to-Park (P2P) - if the contacted station was also at a park
    if qso.pota_ref or qso.sig_info:
        park_ref = qso.pota_ref or qso.sig_info
        fields.append(generate_adif_field("SIG", "POTA"))
        fields.append(generate_adif_field("SIG_INFO", park_ref.upper()))

    # Optional fields
    if qso.name:
        fields.append(generate_adif_field("NAME", qso.name))
    if qso.state:
        fields.append(generate_adif_field("STATE", qso.state))
    if qso.gridsquare:
        fields.append(generate_adif_field("GRIDSQUARE", qso.gridsquare))
    if qso.notes:
        fields.append(generate_adif_field("COMMENT", qso.notes))

    # End of record
    fields.append("<EOR>")

    return " ".join(fields)


def generate_pota_adif(
    qsos: list[QSO],
    my_callsign: str,
    my_park_ref: str,
    my_state: str = "",
    my_grid: str = "",
) -> str:
    """Generate POTA-formatted ADIF data.

    Args:
        qsos: List of QSO objects
        my_callsign: Operator's callsign
        my_park_ref: Park reference being activated
        my_state: Operator's state
        my_grid: Operator's grid square

    Returns:
        POTA-formatted ADIF string
    """
    lines = [generate_adif_header()]

    for qso in qsos:
        lines.append(qso_to_pota_adif(qso, my_callsign, my_park_ref, my_state, my_grid))
        lines.append("")

    return "\n".join(lines)


def get_pota_filename(
    callsign: str,
    park_ref: str,
    activation_date: Optional[datetime] = None,
) -> str:
    """Generate a POTA-compliant filename.

    Format: callsign@park-YYYYMMDD.adi

    Args:
        callsign: Operator's callsign
        park_ref: Park reference (e.g., "K-1234")
        activation_date: Date of activation (defaults to today)

    Returns:
        POTA filename string
    """
    if activation_date is None:
        activation_date = datetime.now(timezone.utc)

    date_str = activation_date.strftime("%Y%m%d")
    # Replace slashes in callsign (portable indicators)
    safe_callsign = callsign.replace("/", "-")
    return f"{safe_callsign}@{park_ref}-{date_str}.adi"


def export_pota_adif(
    qsos: list[QSO],
    filepath: str | Path,
    my_callsign: str,
    my_park_ref: str,
    my_state: str = "",
    my_grid: str = "",
) -> int:
    """Export QSOs to a POTA-formatted ADIF file.

    Args:
        qsos: List of QSO objects to export
        filepath: Path to the output file
        my_callsign: Operator's callsign
        my_park_ref: Park reference being activated
        my_state: Operator's state
        my_grid: Operator's grid square

    Returns:
        Number of QSOs exported
    """
    path = Path(filepath)

    if path.suffix.lower() not in (".adi", ".adif"):
        path = path.with_suffix(".adi")

    adif_data = generate_pota_adif(qsos, my_callsign, my_park_ref, my_state, my_grid)

    with open(path, "w", encoding="utf-8") as f:
        f.write(adif_data)

    return len(qsos)
